''' <summary>
''' definiert eine Flche eines Controls, deren Position und Gre durch ein Rechteck 
''' beschrieben ist und die eigene Funktionen fr Maus-Ereignisse erfllt
''' </summary>
Public Class Area

#Region " Private Felder "
    ' Referenz zur CustomScrollBar
    Private m_ScrollBar As CustomScrollBarBase

    ' Painter der diese Area zeichnet
    Private m_Painter As ScrollBarAreaPainter

    ' Position der Area auf der ScrollBar
    ' fr vertikale Orientierung = Top
    Private m_Position As Integer = 0

    ' Lnge der Area auf der ScrollBar
    ' fr vertikale Orientierung = Height
    Private m_Length As Integer = 0

    ' Flche ist selektiert oder nicht
    Private m_Selected As Boolean = False

    ' Maus befindet sich ber dieser Flche oder nicht
    Private m_Hoovered As Boolean = False

    ' Rechteck, das Position und Gre der Area auf der ScrollBar beschreibt
    Private m_Rectangle As New Rectangle(0, 0, 0, 0)

    ' Flag - wird True, wenn Neuberechnen des Rechtecks erforderlich ist
    Private m_HasRectangleChanges As Boolean = False
#End Region

#Region " Konstruktoren "

    Public Sub New(ByVal ScrollBar As CustomScrollBarBase)
        ' Die Area erhlt eine Referenz zur ScrollBar, um ggf. ein Invalidieren
        ' der Control-Flche vom Area-Painter aus zu ermglichen
        m_ScrollBar = ScrollBar
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Painter, der diese Area zeichnet
    ''' </summary>
    WriteOnly Property Painter() As ScrollBarAreaPainter
        Set(ByVal value As ScrollBarAreaPainter)
            m_Painter = value
            ' Dem Painter eine Referenz zur Area liefern
            m_Painter.Area = Me
        End Set
    End Property

    ''' <summary>
    ''' Rechteck, dass Position und Gre dieser Area auf der ScrollBar wiedergibt
    ''' </summary>
    ReadOnly Property Rectangle() As Rectangle
        Get
            Return m_Rectangle
        End Get
    End Property

    ''' <summary>
    ''' Orientierung der ScrollBar
    ''' </summary>
    ReadOnly Property Orientation() As Orientation
        Get
            Return m_ScrollBar.Orientation
        End Get
    End Property

    ''' <summary>
    ''' Aktivierungszustand der ScrollBar
    ''' </summary>
    ReadOnly Property Enabled() As Boolean
        Get
            Return m_ScrollBar.Enabled
        End Get
    End Property

    ''' <summary>
    ''' Endposition der Area (fr vertikale ScrollBar = Bottom)
    ''' </summary>
    ReadOnly Property Endposition() As Integer
        Get
            Return m_Position + m_Length
        End Get
    End Property

    ''' <summary>
    ''' Information, ob das Rechteck dieser Area neu berechnet werden muss 
    ''' </summary>
    Property HasRectangleChanges() As Boolean
        Get
            Return m_HasRectangleChanges
        End Get
        Set(ByVal value As Boolean)
            m_HasRectangleChanges = value
        End Set
    End Property

    ''' <summary>
    ''' Position der Area (fr vertikale ScrollBar = Top)
    ''' </summary>
    Property Position() As Integer
        Get
            Return m_Position
        End Get
        Set(ByVal value As Integer)
            If m_Position <> value Then
                m_Position = value
                m_HasRectangleChanges = True
            End If
        End Set
    End Property

    ''' <summary>
    ''' Lnge der Area (fr vertikale ScrollBar = Height)
    ''' </summary>
    Property Length() As Integer
        Get
            Return m_Length
        End Get
        Set(ByVal value As Integer)
            If m_Length <> value Then
                m_Length = value
                m_HasRectangleChanges = True
            End If
        End Set
    End Property

    ''' <summary>
    ''' berechnet das Rechteck dieser Area auf der ScrollBar neu
    ''' </summary>
    Private Sub GetRectangle()
        If m_ScrollBar.Orientation = Orientation.Vertical Then
            m_Rectangle = New Rectangle(0, m_Position, m_ScrollBar.VirtualWidth, m_Length)
        Else
            m_Rectangle = New Rectangle(m_Position, 0, m_Length, m_ScrollBar.VirtualWidth)
        End If
    End Sub

    ''' <summary>
    ''' True wenn Maus sich ber dieser Area befindet
    ''' </summary>
    Property Hoovered() As Boolean
        Get
            Return m_Hoovered
        End Get
        Set(ByVal value As Boolean)
            If m_Hoovered <> value Then
                m_Hoovered = value
                ' je nach aktivem Status die OnMouseEnter oder OnMouseLeave-Methode
                ' im AreaPainter aufrufen
                If m_Hoovered Then
                    m_Painter.OnMouseEnter()
                Else
                    m_Painter.OnMouseLeave()
                End If
                ' Neuzeichnen der Area-Zeichenflche auf der ScrollBar erzwingen 
                m_ScrollBar.Invalidate(m_Painter.PaintRectangle)
            End If
        End Set
    End Property


    ''' <summary>
    ''' True, wenn diese Area mit der Maus selektiert wurde
    ''' </summary>
    Property Selected() As Boolean
        Get
            Return m_Selected
        End Get
        Set(ByVal value As Boolean)
            If m_Selected <> value Then
                m_Selected = value
                ' je nach aktivem Status die OnMouseEnter oder OnMouseLeave-Methode
                ' im AreaPainter aufrufen
                If m_Selected Then
                    m_Painter.OnMouseDown()
                Else
                    m_Painter.OnMouseUp()
                End If
                ' Neuzeichnen der Area-Zeichenflche auf der ScrollBar erzwingen 
                m_ScrollBar.Invalidate(m_Painter.PaintRectangle)
            End If
        End Set
    End Property
#End Region

#Region " Methoden "

    ''' <summary>
    ''' bewirkt Aktualisieren des zugeordneten AreaPainters
    ''' </summary>
    Public Sub UpdatePainter()
        ' Rechteck neu berechnen
        GetRectangle()
        ' Painter aktualisieren
        m_Painter.Update()
        ' nderungs-Flag auf False setzen
        m_HasRectangleChanges = False
    End Sub

    ''' <summary>
    ''' ermglicht dem AreaPainter, ein Neuzeichnen eines Rechtecks 
    ''' auf der ScrollBar zu erzwingen
    ''' </summary>
    Public Sub Invalidate(ByVal Rect As Rectangle)
        ' erzwingt Neuzeichnen des Rechtecks auf der ScrollBar
        m_ScrollBar.Invalidate(Rect)
    End Sub
#End Region

End Class
