Imports System.Drawing.Drawing2D
''' <summary>
''' Zeichnet den Hintergrund einer GlassScrollBar
''' </summary>

Public Class GlassScrollBarBackgroundPainter
    Inherits ScrollBarAreaPainter

#Region " Private Felder / Events "

    ' Aktive Hintergrundfarbe
    Private m_ActiveBackColor As Color

    ' Basisfarben fr Gradienten
    Private m_LeftGradientStartColor As Color
    Private m_RightGradientEndColor As Color

    ' Pens zum Zeichnen von Linien
    Private m_LeftPen As Pen
    Private m_BottomPen As Pen

    ' Brushes zum Zeichnen von Gradienten
    Private m_LeftgradientBrush As LinearGradientBrush
    Private m_RightgradientBrush As LinearGradientBrush

    ' Rechtecke zum Zeichnen von Gradienten
    Private m_LeftRect As Rectangle
    Private m_RightRect As Rectangle

    ' Farbensammlung
    Private WithEvents m_Colors As ScrollBarColorCollection

    ' Diese Events werden gebraucht, um dem ScrollBarPainter zu ermglichen,
    ' die Pfeil-Buttons ein- und auszublenden
    Public Event MouseEnter()
    Public Event MouseLeave()

#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor fr einen GlassScrollBarBackgroundPainter
    ''' </summary>
    ''' <param name="Colors">Farbensammlung</param>
    Public Sub New(ByVal Colors As ScrollBarColorCollection)
        m_Colors = Colors
        CreateOwnBrushes(m_Colors.BackColor)
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Zeichenrechteck
    ''' </summary>
    ''' <remarks>Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
    ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
    Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
        Get
            Return Me.Area.Rectangle
        End Get
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
    ''' </summary>
    Public Overrides Sub OnMouseDown()

    End Sub

    ''' <summary>
    ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
    ''' </summary>
    Public Overrides Sub OnMouseEnter()
        CreateOwnBrushes(ControlPaint.Light(m_Colors.BackColor))
        GetRectangleBrushes()
        RaiseEvent MouseEnter()
    End Sub

    ''' <summary>
    ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
    ''' </summary>
    Public Overrides Sub OnMouseLeave()
        CreateOwnBrushes(m_Colors.BackColor)
        GetRectangleBrushes()
        RaiseEvent MouseLeave()
    End Sub

    ''' <summary>
    ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
    ''' </summary>
    Public Overrides Sub OnMouseUp()

    End Sub

    ''' <summary>
    ''' zeichnet die ScrollBarArea neu
    ''' </summary>
    Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        Dim g As Graphics = e.Graphics

        ' Flche mit aktiver Hintergrundfarbe fllen 
        g.Clear(m_ActiveBackColor)

        Dim Rect As Rectangle = Me.Area.Rectangle

        If Me.Area.Orientation = Orientation.Vertical Then
            ' vertikale Orientierung
            g.DrawLine(m_LeftPen, 0, 0, 0, Rect.Height)
            g.FillRectangle(m_LeftgradientBrush, m_LeftRect)
            g.FillRectangle(m_RightgradientBrush, m_RightRect)
            g.DrawLine(m_BottomPen, 1, Rect.Height - 1, Rect.Width - 1, Rect.Height - 1)
        Else
            g.DrawLine(m_LeftPen, 0, 0, Rect.Width, 0)
            ' horizontale Orientierung
            g.FillRectangle(m_LeftgradientBrush, m_LeftRect)
            g.FillRectangle(m_RightgradientBrush, m_RightRect)
            g.DrawLine(m_BottomPen, Rect.Width - 1, 1, Rect.Width - 1, Rect.Height - 1)
        End If
    End Sub

    ''' <summary>
    ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
    ''' </summary>
    Public Overrides Sub Update()
        GetRectangleBrushes()
    End Sub

    ''' <summary>
    ''' Eventhandler fr das Changed-Event der Farbensammlung
    ''' </summary>
    Private Sub m_Colors_Changed() Handles m_Colors.Changed
        CreateOwnBrushes(m_Colors.BackColor)
        GetRectangleBrushes()
    End Sub

    ''' <summary>
    ''' erstellt neue Basisfarben, Pens und Brushes fr eine Hintergundfarbe
    ''' </summary>
    ''' <param name="BaseColor">Color-Struktur</param>
    Private Sub CreateOwnBrushes(ByVal BaseColor As Color)
        m_ActiveBackColor = BaseColor
        Dim Original As New HSLColor(BaseColor)
        Dim HSL As New HSLColor(Original)
        HSL.Lightness += 3.0R / 100.0R
        Dim LightColor As Color = HSL.ToRGB

        HSL = New HSLColor(Original)
        HSL.Lightness -= 5.0R / 100.0R
        m_LeftGradientStartColor = HSL.ToRGB


        HSL = New HSLColor(Original)
        HSL.Lightness -= 2.0R / 100.0R
        m_RightGradientEndColor = HSL.ToRGB

        m_LeftPen = New Pen(LightColor)
        m_BottomPen = New Pen(m_RightGradientEndColor)
    End Sub

    ''' <summary>
    ''' erstellt neue Brushes, die von Position und Gre des Zeichenrechtecks
    ''' abhngig sind
    ''' </summary>
    Private Sub GetRectangleBrushes()

        Dim Rect As Rectangle = Me.Area.Rectangle

        If Me.Area.Orientation = Orientation.Vertical Then
            ' vertikale Orientierung
            m_LeftRect = New Rectangle(1, 0, 3, Rect.Height)
            m_LeftgradientBrush = New LinearGradientBrush(m_LeftRect, m_LeftGradientStartColor, m_ActiveBackColor, LinearGradientMode.Horizontal)

            m_RightRect = New Rectangle(Rect.Width - 4, 0, 4, Rect.Height)
            m_RightgradientBrush = New LinearGradientBrush(m_RightRect, m_ActiveBackColor, m_RightGradientEndColor, LinearGradientMode.Horizontal)

        Else
            ' horizontale Orientierung
            m_LeftRect = New Rectangle(0, 1, Rect.Width, 3)
            m_LeftgradientBrush = New LinearGradientBrush(m_LeftRect, m_LeftGradientStartColor, m_ActiveBackColor, LinearGradientMode.Vertical)

            m_RightRect = New Rectangle(0, Rect.Height - 4, Rect.Width, 4)
            m_RightgradientBrush = New LinearGradientBrush(m_RightRect, m_ActiveBackColor, m_RightGradientEndColor, LinearGradientMode.Vertical)

        End If
    End Sub

#End Region

End Class
