Imports System.Collections.ObjectModel
Imports System.Windows.Forms
''' <summary>
''' zentrale Verwaltungsstelle fr Dateien und Forms fr's Multi-SDI-Modell
''' </summary>
Public Class FormsManager

#Region " Private Felder "

    ' Dictionary mit allen offenen MainForms
    Private m_OpenFiles As New Dictionary(Of Form, Datei)
    ' Liste mit offenen Forms
    Private m_OpenForms As New List(Of Form)

    ' Eigenschaften des verwendeten Datei-Typs
    Private m_Def As DateiDefinition
    ' Zhler fr neue Dokumente
    Private m_DocumentCounter As Integer = 0

    ' Flag, die True ist, wenn wegen Cancel im Speichern-Dialog "Alles schliessen" eine Form
    ' nicht geschlossen werden konnte
    Private m_CloseAllCanceled As Boolean = False

    ' Die aktive Form der Anwendung mit eigener Datei
    Private m_ActiveForm As Form

#End Region

#Region " Properties "

    ''' <summary>
    ''' Globaler Anwendungs-Titel
    ''' </summary>
    ReadOnly Property AnwendungsTitel() As String
        Get
            Return m_Def.AppTitle
        End Get
    End Property

    ''' <summary>
    ''' Liste mit geffneten Windows Forms
    ''' </summary>
    ReadOnly Property OpenForms() As List(Of Form)
        Get
            Return m_OpenForms
        End Get
    End Property
#End Region

#Region " Konstruktor "

    ''' <summary>
    ''' Konstruktor mit fester Datei-Definition
    ''' </summary>
    ''' <param name="Def">Datei-Definition</param>
    Public Sub New(ByVal Def As DateiDefinition)
        m_Def = Def

        ' DateiDefinition soll konstruktorlos bleiben, um die Implementierung 
        ' zu erleichtern. Also erstelle ich die notwendigen gebeugten Meldungs-Inhalte hier.
        Select Case m_Def.Geschlecht
            Case Geschlechter.maennlich
                m_Def.Txt_diegeoeffneteDatei = "den geffneten " & m_Def.DocumentName
                m_Def.Txt_DieseDatei = "Diesen " & m_Def.DocumentName
                m_Def.Txt_eineDatei = "ein " & m_Def.DocumentName
                m_Def.Txt_sie = "ihn"
            Case Geschlechter.weiblich
                m_Def.Txt_diegeoeffneteDatei = "die geffnete " & m_Def.DocumentName
                m_Def.Txt_DieseDatei = "Diese " & m_Def.DocumentName
                m_Def.Txt_eineDatei = "eine " & m_Def.DocumentName
                m_Def.Txt_sie = "sie"
            Case Geschlechter.saechlich
                m_Def.Txt_diegeoeffneteDatei = "das geffnete " & m_Def.DocumentName
                m_Def.Txt_DieseDatei = "Dieses " & m_Def.DocumentName
                m_Def.Txt_eineDatei = "ein " & m_Def.DocumentName
                m_Def.Txt_sie = "es"
        End Select
    End Sub

#End Region

#Region " Methoden "

#Region " Dateien ffnen und speichern "

    ''' <summary>
    ''' liefert die einer Windows Form zugeordnete Datei
    ''' </summary>
    ''' <param name="frm">Windows Form</param>
    Public Function GetFile(ByVal frm As Form) As Datei
        If m_OpenFiles.ContainsKey(frm) Then
            Return m_OpenFiles(frm)
        Else
            Return Nothing
        End If
    End Function

    ''' <summary>
    ''' registriert eine Windows Form und erstellt eine neue Datei mit Hilfe von CommandLineArgs
    ''' </summary>
    ''' <param name="Frm">Windows Form</param>
    ''' <param name="CommandLineArgs">CommandLineArgs</param>
    Public Sub RegisterForm(ByVal Frm As Form, ByVal CommandLineArgs As ReadOnlyCollection(Of String))
        If Not m_OpenFiles.ContainsKey(Frm) Then
            ' nur wenn die Instanz der Form nicht vorher schon registriert war
            ' Pfad der Datei aus CommandLineArgs besorgen
            Dim Path As String = GetCommandLine(CommandLineArgs)
            ' neue Datei mit Pfad erstellen
            Dim NewFile As Datei = CreateNewFile(Path)
            If NewFile Is Nothing Then
                ' Wenn z.B. der Dialog zum Erstellen einer schreibgeschtzten Kopie 
                ' abgebrochen wird, wird dazu auch keine Windows Form angezeigt
                Frm.Close()
            Else
                ' Datei und Windows Form zu internen Auflistungen hinzu fgen und mit Eventhandlern versehen
                AddToOpenForms(Frm, NewFile)
            End If
        End If
    End Sub

    ''' <summary>
    ''' liefert eine Windows Form mit Datei fr einen angegebenen Datei-Pfad
    ''' </summary>
    ''' <param name="Path">Pfad der Datei (kann leer sein)</param>
    ''' <remarks>bei leerem Pfad wird eine neue Datei erstellt</remarks>
    Private Function CreateNewFile(ByVal Path As String) As Datei
        ' ist eigentlich nur zur Verwendung in ApplicationEvents vorgesehen
        ' und liefert eine Form entweder beim Start der Anwendung 
        ' oder bei Doppelklick auf eine Datei (Pfad aus CommandLineArgs)

        If Path = "" Then
            Return CreateNewDatei("", False)
        Else
            Dim IsFileOpen As Boolean = False
            Dim LowerPath As String = Path.ToLower

            ' Herausfinden, ob eine Datei mit gleichem Dateinamen schon offen ist
            For Each frm As Form In m_OpenForms
                Dim File As Datei = m_OpenFiles(frm)
                If File.Pfad.ToLower = LowerPath AndAlso (Not File.IsReadOnly) Then
                    IsFileOpen = True
                    Exit For
                End If
            Next

            If IsFileOpen Then
                ' Wenn eine datei mit gleichem Namen bereits geffnet ist
                Dim Result As DialogResult = MessageBox.Show(m_Def.Txt_DieseDatei & " ist bereits geffnet." & vbCrLf & "Mchten Sie " & m_Def.Txt_sie & " als schreibgeschtzte Kopie ffnen?", m_Def.AppTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question)
                Select Case Result
                    Case DialogResult.Yes
                        ' neue schreibgeschtzte Datei erstellen
                        Return CreateNewDatei(Path, True)
                    Case DialogResult.No
                        ' nichts zu tun
                End Select
            Else
                ' wenn noch keine Datei mit gleichem Namen offen ist, kann sie erstellt werden
                Return CreateNewDatei(Path, False)
            End If
        End If

        Return Nothing
    End Function

    ''' <summary>
    ''' liefert eine Datei fr den angegebenen Pfad
    ''' </summary>
    ''' <param name="Path">Pfad (kann leer sein)</param>
    ''' <param name="IsReadOnly">Read-Only-Eigenschaft</param>
    Private Function CreateNewDatei(ByVal Path As String, ByVal IsReadOnly As Boolean) As Datei
        ' hier wird schon viel Code ausgefhrt, der eventuell Fehler enthlt

        ' Wenn ein Fehler auftritt, soll dieser nicht zum Absturz der gesamten Anwendung fhren
        ' Also wird hier einf Fehlerbehandlung eingefgt

        Dim NewFile As Datei = Nothing

        Try
            ' Datei (bei leerem Pfad neue Datei) erstellen
            NewFile = m_Def.CreateFile(Path)
            ' ReadOnly vermerken
            NewFile.IsReadOnly = IsReadOnly
            ' wenn es eine neue Datei ist, Fenstername z.B. als "Dokument1" angeben
            If Path = "" Then
                m_DocumentCounter += 1
                NewFile.DisplayName = m_Def.DocumentName & m_DocumentCounter.ToString
            End If
            ' Datei fr spteren Vergleich zwischenspeichern
            NewFile.Zustand_Speichern()
        Catch ex As Exception
            ' bei Fehler Fehlermeldung
            MessageBox.Show(ex.Message, m_Def.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Error)
            If NewFile IsNot Nothing Then
                Try
                    ' versuchen, den temporren Inhalt der Datei freizugeben
                    NewFile.Dispose()
                Catch ex1 As Exception

                End Try
            End If
        End Try

        ' Datei zurckgeben
        Return NewFile
    End Function

    ''' <summary>
    ''' liefert den Startpfad fr eine Datei aus der CommandLine
    ''' </summary>
    Private Function GetCommandLine(ByVal args As ReadOnlyCollection(Of String)) As String
        Dim Result As String = ""
        Dim Line As String = ""
        For Each txt As String In args
            Line &= " " & txt
        Next
        Line = Line.Trim

        If Line.IndexOf("/open/") > -1 Then
            Result = GetLongFilename(Line.Substring(0, Line.IndexOf("/open/"))).Trim
        End If
        Return Result
    End Function

    ''' <summary>
    ''' wandelt kurze Dateinamen in lange um
    ''' </summary>
    ''' <param name="ShortName">(mglicherweise kurzer) Dateiname</param>
    Private Function GetLongFilename(ByVal ShortName As String) As String
        Try
            Return New IO.FileInfo(ShortName).FullName
        Catch ex As Exception

        End Try
        Return ""
    End Function

    ''' <summary>
    ''' Speichert die Datei und liefert bei erfolgreichem Abschluss True
    ''' </summary>
    Public Function Save() As Boolean
        ' Referenz zur Datei
        Dim File As Datei = m_OpenFiles(m_ActiveForm)

        ' Wenn die Datei gendert wurde
        If File.Veraendert Then
            ' Wenn die Datei einen bekannten Pfad hat und nicht ReadOnly ist
            If File.Pfad <> "" AndAlso (Not File.IsReadOnly) Then
                ' versuchen unter diesem Pfad zu speichern
                Try
                    Dim Result As Boolean = File.Speichern(File.Pfad)
                    If Result = True Then
                        File.Zustand_Speichern()
                    End If
                    Return Result
                Catch ex As Exception
                    MessageBox.Show(ex.Message, m_Def.AppTitle & " - Fehler beim Speichern", MessageBoxButtons.OK)
                End Try
                ' bei Misserfolg False zurck geben
                Return False
            Else
                ' Wenn die Datei neu ist (keinen Pfad hat) oder ReadOnly ist, 
                ' Speichern-Unter-Dialog anzeigen
                Return Speichern_Unter_Erfolgreich(File)
            End If
        Else
            ' wenn die Datei gegenber dem letzten Speichern nicht verndert wurde, 
            ' kann True zurck gegeben werden
            Return True
        End If
    End Function

    ''' <summary>
    ''' ffnet einen Speichern-Unter-Dialog fr die Aktive Form 
    ''' und liefert nach erfolgreichem Speichern True
    ''' </summary>
    Public Function Save_As() As Boolean
        ' Referenz zur Datei
        Dim File As Datei = m_OpenFiles(m_ActiveForm)
        Return Speichern_Unter_Erfolgreich(File)

    End Function

    ''' <summary>
    ''' Zeigt den Speichern-Unter-Dialog fr eine Datei 
    ''' und liefert nur nach erfolgreichem Speichern True
    ''' </summary>
    ''' <param name="File">Referenz zu einer Datei</param>
    Private Function Speichern_Unter_Erfolgreich(ByVal File As Datei) As Boolean
        ' spezieller Speichern-Unter-Dialog, der auch bercksichtigt, dass
        ' schreibgeschtzte Dateien nicht ihre Quelle berschreiben drfen
        Dim Dlg As New SpeichernDialog(File, m_Def, AllFiles)

        ' Der Dialog liefert nur Erfolgreich, wenn ein erlaubter Dateiname ausgewhlt wurde
        If Dlg.ShowDialog_Erfolgreich Then
            ' Datei unter dem ausgewhlten Dateinamen speichern
            If File.Speichern(Dlg.FileName) Then
                ' wenn Speichern erfolgreich: 
                ' Datei erhlt neuen Dateinamen 
                File.Pfad = Dlg.FileName
                ' Datei ist nicht mehr ReadOnly (wenn sie es zuvor gewesen wre)
                File.IsReadOnly = False
                ' Aktuellen Zustand der Datei temporr zwischenspeichern
                File.Zustand_Speichern()
                ' Speicern war erfolgreich
                Return True
            End If
        End If
        ' in allen anderen Fllen False zurck geben
        Return False
    End Function

    ''' <summary>
    ''' sorgt beim Schlieen einer Windows Form fr Speichern der Datei
    ''' </summary>
    ''' <param name="File">Referenz zur Datei</param>
    ''' <remarks>Diese Funktion wird nur beim Schlieen einer Windows Form verwendet .
    ''' Die Funktion liefert nur False, wenn das Speichern abgebrochen wird.
    ''' Ansonsten wird entweder gespeichert oder darauf verzichtet</remarks>
    Private Function SpeichernMitFrageErfolgreich(ByVal File As Datei) As Boolean
        ' nur wenn die Datei verndert wurde
        If File.Veraendert Then
            ' Rckfrage, ob nderungen gespeichert werden sollen
            Dim Result As DialogResult = MessageBox.Show("nderungen in " & File.FormText & " speichern?", _
            m_Def.AppTitle, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question)

            If Result = Windows.Forms.DialogResult.Yes Then
                ' wenn gespeichert werden soll
                If File.Pfad <> "" AndAlso (Not File.IsReadOnly) Then
                    ' Wenn die Datei schon mal gespeichert war und sie nicht ReadOnly ist,
                    ' versuchen sie ganz normal zu speichern
                    Try
                        Return File.Speichern(File.Pfad)
                    Catch ex As Exception
                        MessageBox.Show(ex.Message, m_Def.AppTitle & " - Fehler beim Speichern", MessageBoxButtons.OK)
                    End Try
                    Return False
                Else
                    ' Wenn die Datei noch keinen Pfad hat, also neu ist, oder schreibgeschtzt,
                    ' Speichern-Unter-Dialog anzeigen
                    Return Speichern_Unter_Erfolgreich(File)
                End If
            ElseIf Result = Windows.Forms.DialogResult.No Then
                ' Wenn nach der Antwort im Dialog nicht gespeicert werden soll,
                ' wird True zurck gegeben
                Return True
            ElseIf Result = Windows.Forms.DialogResult.Cancel Then
                ' wenn der Speichern-Dialog abgebrochen wurde, wird False zurck gegeben
                Return False
            End If
        Else
            ' Wenn die Datei in ihrem Original-Zustand ist, braucht nicht gespeichert werden,
            ' also kann True zurck gegeben werden.
            Return True
        End If
    End Function

    ''' <summary>
    ''' ffnet eine Datei mit Dialogfeld
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Open_Dialog()
        ' Referenz zur aktiven Form
        Dim oldfrm As Form = m_ActiveForm
        Dim OldFile As Datei = m_OpenFiles(oldfrm)
        ' Spezieller Dialog zum Datei-ffnen unter Bercksichtigung, dass schon offene 
        ' Dateien nur schreibgeschtzt ein zweites Mal geffnet werden drfen
        Dim ODlg As New OeffnenDialog(m_Def, AllFiles)
        ' nur wenn in diesem Dialog eine Datei erfolgreich ausgewhlt wurde
        ' (oder besttigt wurde, dass schreibgeschtzt geffnet werden darf)
        If ODlg.ShowDialog_Erfolgreich Then
            ' Form zur Datei anzeigen
            Dim NewForm As Form = m_Def.CreateForm
            Dim NewDatei As Datei = Me.CreateNewDatei(ODlg.FileName, ODlg.FileAlreadyOpen)
            ' Datei und Windows Form zu internen Auflistungen hinzu fgen und mit Eventhandlern versehen
            AddToOpenForms(NewForm, NewDatei)
            NewForm.Show()

            ' wenn die Form, von der aus eine neue Datei geffnet wurde, leer war,
            ' kann diese Form geschlossen werden
            If OldFile.IsEmpty AndAlso (Not OldFile.Veraendert) Then
                ' neue form erscheint am Platz der alten
                NewForm.Location = oldfrm.Location
                NewForm.Size = oldfrm.Size
                oldfrm.Close()
            End If
        End If
    End Sub

    ''' <summary>
    ''' Erstellt eine leere Datei und ffnet die dazu gehrende Windows Form
    ''' </summary>
    Public Sub Create_New()
        ' neue Windows Form erstellen
        Dim NewForm As Form = m_Def.CreateForm
        ' neue leere Datei erstellen
        Dim NewDatei As Datei = Me.CreateNewDatei("", False)
        ' Datei und Windows Form zu internen Auflistungen hinzu fgen und mit Eventhandlern versehen
        AddToOpenForms(NewForm, NewDatei)
        ' neue Windows Form anzeigen
        NewForm.Show()
    End Sub

    ''' <summary>
    ''' Schliesst alle offenen Windows Forms
    ''' </summary>
    Public Sub CloseAllForms()
        m_CloseAllCanceled = False

        ' eine Liste mit genderten und eine mit unvernderten Windows Forms erstellen
        Dim ChangedForms As New List(Of Form)
        Dim NotchangedForms As New List(Of Form)

        For Each frm As Form In m_OpenForms
            If m_OpenFiles(frm).Veraendert Then
                ChangedForms.Add(frm)
            Else
                NotchangedForms.Add(frm)
            End If
        Next

        ' Die Liste mit vernderten Forms zuerst durchgehen, und versuchen,
        ' sie zu schlieen
        For Each frm As Form In ChangedForms
            frm.Close()
            ' wenn es hier wegen Cancel beim Speichern nicht gelang, 
            ' das Fenster zu schlieen, wird die Schleife abgebrochen
            If m_CloseAllCanceled = True Then
                Exit For
            End If
        Next

        ' Wenn der Benutzer Speichern durch "Abbrechen" unterbrochen hat,
        ' bleiben die unvernderten Forms auch offen
        If Not m_CloseAllCanceled Then
            For Each frm As Form In NotchangedForms
                frm.Close()
            Next
        End If

    End Sub

    ''' <summary>
    ''' liefert eine Liste mit allen geffneten Dateien
    ''' </summary>
    Private Function AllFiles() As List(Of Datei)
        Dim Result As New List(Of Datei)
        For Each frm As Form In m_OpenForms
            Result.Add(m_OpenFiles(frm))
        Next
        Return Result
    End Function

#End Region

#Region " Eventhandler "

    ''' <summary>
    ''' Fgt eine neue Windows Form mit Datei den internen Auflistungen hinzu
    ''' und ordnet der Form notwendige Eventhandler zu
    ''' </summary>
    ''' <param name="frm">anzuzeigende Form</param>
    Private Sub AddToOpenForms(ByVal frm As Form, ByVal File As Datei)
        ' Form und Datei zu Auflistungen hinzufgen
        m_OpenFiles.Add(frm, File)
        m_OpenForms.Add(frm)

        ' verschiedene Handler fr Ereignisse der Windows Form, die hier bentigt werden
        AddHandler frm.Activated, AddressOf Form_Activated
        AddHandler frm.FormClosed, AddressOf Form_FormClosed
        AddHandler frm.FormClosing, AddressOf FormClosing

    End Sub

    ''' <summary>
    ''' Eventhandler fr das FormClosed-Ereignis der Windows Forms
    ''' </summary>
    Private Sub Form_FormClosed(ByVal sender As Object, ByVal e As FormClosedEventArgs)
        ' Wenn eine Windows Form geschlossen wurde,
        ' wird falls noch vorhanden, die erste Form in der OpenForms-Auflistung 
        ' zur MainForm
        Dim frm As Form = CType(sender, Form)
        ' Datei bereinigen
        m_OpenFiles(frm).Dispose()
        ' Form und Datei aus Auflistungen entfernen
        m_OpenFiles.Remove(frm)
        m_OpenForms.Remove(frm)

        RemoveHandler frm.Activated, AddressOf Form_Activated
        RemoveHandler frm.FormClosed, AddressOf Form_FormClosed
        RemoveHandler frm.FormClosing, AddressOf FormClosing
    End Sub

    ''' <summary>
    ''' Eventhandler fr das Activated-Ereignis der Windows Forms
    ''' </summary>
    Private Sub Form_Activated(ByVal sender As Object, ByVal e As EventArgs)
        ' Bei Aktivierung einer Datei-Form wird diese zur MainForm
        m_ActiveForm = CType(sender, Form)
        m_OpenForms.Remove(m_ActiveForm)
        m_OpenForms.Insert(0, m_ActiveForm)
    End Sub



    ''' <summary>
    ''' Eventhandler fr das FormClosing-Ereignis der Windows Forms
    ''' </summary>
    Private Sub FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs)
        ' sorgt dafr, dass beim Schliessen einer Datei-Form gespeichert wird oder nicht
        Dim frm As Form = CType(sender, Form)
        Dim File As Datei = m_OpenFiles(frm)
        ' Wenn eine Datei-Form geschlossen werden soll, und die Datei verndert wurde,
        If File.Veraendert Then
            ' Form aktivieren
            frm.Activate()
            ' Versuchen, die Datei mit Rckfrage zu speichern.
            ' Die SpeichernMitFrageErfolgreich liefert True, wenn erfolgreich gespeichert
            ' wurde oder auf Speichern verzichtet werden soll.
            If Not SpeichernMitFrageErfolgreich(File) Then
                ' wenn das Speichern abgebrochen wurde:
                ' Flag auf True setzen
                m_CloseAllCanceled = True
                ' Die Windows Form erhlt fr das Schlieen-Ereigns eine Cancel-Information
                e.Cancel = True
            Else
                ' Flag auf False setzen und Datei-Ressourcen frei geben
                m_CloseAllCanceled = False
            End If
        Else
            ' Flag auf False setzen und Datei-Ressourcen frei geben
            m_CloseAllCanceled = False
        End If
    End Sub



#End Region

#Region " Registry-Zugriff "

    ''' <summary>
    ''' Registriert einen Datei-Typ
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub RegisterFileType(ByVal ApplicationStartPath As String)
        ' Argumente zur bergabe an zweite Exe, die mit Administrator-Rechten den Zugriff 
        ' auf die Registry erledigt. Brim Start dieser Exe werden die fr einen Registry-Eintrag
        ' notwendigen Informationen als CommandLineArgs bergeben
        Dim args As String = ""
        ' Anwendungs-Directory (als Basis fr alle anderen hier vorhandenen Pfadangabe)
        args &= "/*/" & Application.StartupPath
        ' Kurztitel zum Eintrag in der Registry
        args &= "/*/" & m_Def.AppShortTitle
        ' Anwendungs-Titel
        args &= "/*/" & m_Def.AppTitle
        ' Dokument-Nname
        args &= "/*/" & m_Def.DocumentName
        ' Dateiname der Exe (incl. Extension), mit der alle Dokumente geffnet werden sollen
        args &= "/*/" & IO.Path.GetFileName(Application.ExecutablePath)
        ' Dateiname (incl. Extension) des Icons, das fr Dokumente verwendet werden soll
        args &= "/*/" & m_Def.IconFileName
        ' Standard-Extension fr Dokumente des eigenen Datei-Typs
        args &= "/*/" & m_Def.Extension

        ' relativen Pfad zur Registrierungs-Exe ermitteln
        'Dim DI As New IO.DirectoryInfo(ApplicationStartPath)
        ' Versuchen, die Registrierungs-Exe mit Admin-Rechten und hier zusammengefassten 
        ' CommandLineArgs auszufhren
        StartElevated(ApplicationStartPath & "\DateiTypRegistrierer.exe", "", args)
    End Sub

    ''' <summary>
    ''' ermglicht, einen Prozess nach Benutzer-Zustimmung mit Administrator-Rechten auszufhren
    ''' </summary>
    ''' <param name="Filename">Dateiname</param>
    ''' <param name="WorkingDirectory">Arbeitspfad</param>
    ''' <param name="Arguments">zustzliche Argumente</param>
    Private Sub StartElevated(ByVal Filename As String, ByVal WorkingDirectory As String, ByVal Arguments As String)
        ' Der Elevation-Vorgang wird von der Shell gesteuert.

        Dim startInfo As New ProcessStartInfo()

        ' wenn Verb = "runas" wird der Consent-Dialog aufgerufen
        startInfo.Verb = "runas"

        startInfo.WorkingDirectory = WorkingDirectory
        startInfo.FileName = Filename
        startInfo.Arguments = Arguments

        Try
            ' versuchen, neuen Prozess mit Administrator-Rechten zu starten
            Process.Start(startInfo)
        Catch ex As Exception
            ' wenn der Benutzer ablehnt, wird hier ein Fehler abgefangen
            MessageBox.Show(ex.Message)
        End Try
    End Sub

#End Region

#End Region

End Class
