Public Class Rect

    ''' <summary>
    ''' Liefert ein zentriertes Rechteck fr gewnschte Breite und Hhe in einem Zielbereich mit Breiten- und Hhenangabe
    ''' </summary>
    ''' <param name="DestinationWidth">Breite des Zielbereichs</param>
    ''' <param name="DestinationHeight">Hhe des Zielbereichs</param>
    ''' <param name="OriginalWidth">gewnschte Breite des Rechtecks</param>
    ''' <param name="OriginalHeight">gewnschte Hhe des Rechtecks</param>
    ''' <returns>Rectangle</returns>
    Public Shared Function GetCenterRect(ByVal DestinationWidth As Integer, ByVal DestinationHeight As Integer, ByVal OriginalWidth As Integer, ByVal OriginalHeight As Integer) As Rectangle

        Dim pw As Decimal = OriginalWidth
        Dim ph As Decimal = OriginalHeight
        Dim dw As Decimal = DestinationWidth
        Dim dh As Decimal = DestinationHeight

        Dim x As Decimal = (dw - pw) / 2D
        Dim y As Decimal = (dh - ph) / 2D

        Dim Left, Top, Width, Height As Integer
        Left = Math.Round(x)
        Top = Math.Round(y)
        Width = DestinationWidth - Left * 2
        Height = DestinationHeight - Top * 2

        Return New Rectangle(Left, Top, Width, Height)

    End Function

    ''' <summary>
    ''' Liefert ein passend maximiertes Rechteck fr gewnschte Breite und Hhe in einem Zielbereich mit Breiten- und Hhenangabe
    ''' </summary>
    ''' <param name="DestinationWidth">Breite des Zielbereichs</param>
    ''' <param name="DestinationHeight">Hhe des Zielbereichs</param>
    ''' <param name="OriginalWidth">gewnschte Breite des Rechtecks</param>
    ''' <param name="OriginalHeight">gewnschte Hhe des Rechtecks</param>
    ''' <returns>Rectangle</returns>
    Public Shared Function GetMatchRect(ByVal DestinationWidth As Integer, ByVal DestinationHeight As Integer, ByVal OriginalWidth As Integer, ByVal OriginalHeight As Integer) As Rectangle

        Dim pw As Decimal = OriginalWidth
        Dim ph As Decimal = OriginalHeight
        Dim dw As Decimal = DestinationWidth
        Dim dh As Decimal = DestinationHeight

        Dim scaleFactor As Decimal
        If pw / ph > dw / dh Then
            scaleFactor = dw / pw
        Else
            scaleFactor = dh / ph
        End If

        Dim x As Decimal = (dw - scaleFactor * pw) / 2D
        Dim y As Decimal = (dh - scaleFactor * ph) / 2D

        Dim Left, Top, Width, Height As Integer

        Left = Math.Round(x)
        Top = Math.Round(y)
        Width = DestinationWidth - Left * 2
        Height = DestinationHeight - Top * 2

        Return New Rectangle(Left, Top, Width, Height)

    End Function
    ''' <summary>
    ''' Liefert ein zentriertes Rechteck fr gewnschte Breite und Hhe in einem Zielbereich mit Breiten- und Hhenangabe
    ''' </summary>
    ''' <param name="DestinationWidth">Breite des Zielbereichs</param>
    ''' <param name="DestinationHeight">Hhe des Zielbereichs</param>
    ''' <param name="OriginalWidth">gewnschte Breite des Rechtecks</param>
    ''' <param name="OriginalHeight">gewnschte Hhe des Rechtecks</param>
    ''' <returns>RectangleF</returns>
    Public Shared Function GetCenterRect(ByVal DestinationWidth As Single, ByVal DestinationHeight As Single, ByVal OriginalWidth As Single, ByVal OriginalHeight As Single) As RectangleF

        Dim pw As Decimal = OriginalWidth
        Dim ph As Decimal = OriginalHeight
        Dim dw As Decimal = DestinationWidth
        Dim dh As Decimal = DestinationHeight

        Dim x As Decimal = (dw - pw) / 2D
        Dim y As Decimal = (dh - ph) / 2D

        Dim Left, Top, Width, Height As Single
        Left = CSng(x)
        Top = CSng(y)
        Width = DestinationWidth - Left * 2.0F
        Height = DestinationHeight - Top * 2.0F

        Return New RectangleF(Left, Top, Width, Height)

    End Function

    ''' <summary>
    ''' Liefert ein passend maximiertes Rechteck fr gewnschte Breite und Hhe in einem Zielbereich mit Breiten- und Hhenangabe
    ''' </summary>
    ''' <param name="DestinationWidth">Breite des Zielbereichs</param>
    ''' <param name="DestinationHeight">Hhe des Zielbereichs</param>
    ''' <param name="OriginalWidth">gewnschte Breite des Rechtecks</param>
    ''' <param name="OriginalHeight">gewnschte Hhe des Rechtecks</param>
    ''' <returns>RectangleF</returns>
    Public Shared Function GetMatchRect(ByVal DestinationWidth As Single, ByVal DestinationHeight As Single, ByVal OriginalWidth As Single, ByVal OriginalHeight As Single) As RectangleF

        Dim pw As Decimal = OriginalWidth
        Dim ph As Decimal = OriginalHeight
        Dim dw As Decimal = DestinationWidth
        Dim dh As Decimal = DestinationHeight

        Dim scaleFactor As Decimal
        If pw / ph > dw / dh Then
            scaleFactor = dw / pw
        Else
            scaleFactor = dh / ph
        End If

        Dim x As Decimal = (dw - scaleFactor * pw) / 2D
        Dim y As Decimal = (dh - scaleFactor * ph) / 2D

        Dim Left, Top, Width, Height As Single

        Left = CSng(x)
        Top = CSng(y)
        Width = DestinationWidth - Left * 2.0F
        Height = DestinationHeight - Top * 2.0F

        Return New RectangleF(Left, Top, Width, Height)
    End Function
End Class
