''' <summary>
''' Hilfsklasse zum Erstellen einer Bitmap, die entweder eine transparente Farbe oder eine 
''' nichttransparente Farbe auf einem Button anzeigt
''' </summary>
Public Class GifBackColorColorBitmapCreator

#Region " Felder "
    ' angezeigte Farbe
    Private m_Color As Color
    ' Pen zum Zeichnen eines Rahmens auf der Bitmap
    Private m_BorderPen As Pen
    ' Bitmap
    Private m_Bitmap As Bitmap
#End Region

#Region " Properties "
    ReadOnly Property Bitmap() As Bitmap
        Get
            Return m_Bitmap
        End Get
    End Property
    ''' <summary>
    ''' angezeigte Farbe
    ''' </summary>
    WriteOnly Property Color() As Color
        Set(ByVal value As Color)
            m_Color = value
            m_Bitmap = Nothing
            CreateBitmap()
        End Set
    End Property
    ''' <summary>
    ''' Pen zum Zeichnen eines Rahmens auf der Bitmap
    ''' </summary>
    WriteOnly Property BorderPen() As Pen
        Set(ByVal value As Pen)
            m_BorderPen = value
            RefreshBorder()
        End Set
    End Property
#End Region

#Region " Konstruktoren "
    ''' <summary>
    ''' Konstruktor mit einer Startfarbe und einem Pen zum Zeichnen des Rahmens
    ''' </summary>
    ''' <param name="Col">Startfarbe</param>
    ''' <param name="BorderPen">Pen zum Zeichnen des Rahmens</param>
    Public Sub New(ByVal Col As Color, ByVal BorderPen As Pen)
        m_Color = Col
        m_BorderPen = BorderPen
        ' Bitmap neu erstellen
        CreateBitmap()
    End Sub
#End Region

#Region " Methoden "
    ''' <summary>
    ''' erstellt die Bitmap neu
    ''' </summary>
    Private Sub CreateBitmap()
        ' neue Bitmap mit 16*16 Pixeln und 32 bit Farbtiefe
        m_Bitmap = New Bitmap(16, 16, Imaging.PixelFormat.Format32bppArgb)
        ' Referenz zum Graphics-Objekt der Bitmap
        Using g As Graphics = Graphics.FromImage(m_Bitmap)
            ' auf diesem Graphics-Objekt wird gezeichnet
            g.PageUnit = GraphicsUnit.Pixel
            g.SmoothingMode = Drawing2D.SmoothingMode.None
            ' wenn die anzuzeigende Farbe transparent ist
            If m_Color.A = 0 Then
                ' wird ein Raster aus grauen und weissen Rechtecken angezeigt
                Dim GrayBrush As New SolidBrush(System.Drawing.Color.Gray)
                For I As Integer = 0 To 16 Step 8
                    For J As Integer = 0 To 16 Step 8
                        g.FillRectangle(GrayBrush, New Rectangle(I, J, 4, 4))
                        g.FillRectangle(GrayBrush, New Rectangle(I + 4, J + 4, 4, 4))
                    Next
                Next
                ' Ressourcen fr grauen Brush frei geben
                GrayBrush.Dispose()
            Else
                ' ansonsten nur die Bitmap-Flache mit dem nichttransparenten Anzeige-Farbton fllen
                g.Clear(m_Color)
            End If
            ' Rahmenrechteck zeichnen
            g.DrawRectangle(m_BorderPen, New Rectangle(0, 0, 15, 15))
        End Using
    End Sub

    ''' <summary>
    ''' zeichnet nur den farbigen Rahmen auf der Bitmap neu, die Bitmap wird nicht nochmals neu erstellt
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RefreshBorder()
        Using g As Graphics = Graphics.FromImage(m_Bitmap)
            g.PageUnit = GraphicsUnit.Pixel
            g.SmoothingMode = Drawing2D.SmoothingMode.None
            g.DrawRectangle(m_BorderPen, New Rectangle(0, 0, 15, 15))
        End Using
    End Sub
#End Region

End Class
