Imports Ctls
' Die hier folgenden Commands erhalten Informationen zu nderungen an einer Icon-Datei.
' Die in einer Command-Instanz enthaltenen Felder sollten mglichst wenig Speicher 
' in Anspruch nehmen, aber alle wesentlichen Informationen enthalten, um das Resultat 
' eines Befehls rckgangig zu machen oder wiederherstellen zu knnen.

''' <summary>
''' Command zum Lschen eines IconImage
''' </summary>
Public Class RemoveImageCommand
    Inherits Command

    Private m_Datei As IconDatei    ' Referenz zur Icon-Datei
    Private m_ImagePath As String   ' Temporrer Speicherpfad des IconImage-Objektes, das gelscht wird

    ''' <summary>
    ''' Konstruktor
    ''' </summary>
    ''' <param name="Datei">Referenz zur Icon-Datei</param>
    ''' <param name="ImagePath">Temporrer Speicherpfad des IconImage-Objektes</param>
    Public Sub New(ByVal Datei As IconDatei, ByVal ImagePath As String)
        m_Datei = Datei
        m_ImagePath = ImagePath
    End Sub

    ''' <summary>
    ''' Aktionen bei erstmaligem Ausfhren des Commands
    ''' </summary>
    Public Overrides Sub RunFirst()
        ' Instanz des zu lschenden IconImage-Objektes holen
        Dim Img As IconImage = m_Datei.GetImage(m_ImagePath)
        ' Dieses Objekt aus der Datei entfernen
        m_Datei.Remove(Img)
    End Sub

    ''' <summary>
    ''' Bezeichnung des Befehls, der in der History-Liste angezeigt wird
    ''' </summary>
    Public Overrides ReadOnly Property Name() As String
        Get
            Return "Bild entfernen"
        End Get
    End Property

    ''' <summary>
    ''' Aktionen zum Wiederherstellen
    ''' </summary>
    Public Overrides Sub Redo()
        ' Instanz des zu lschenden IconImage-Objektes holen
        Dim Img As IconImage = m_Datei.GetImage(m_ImagePath)
        ' Dieses Objekt aus der Datei entfernen
        m_Datei.Remove(Img)
    End Sub

    ''' <summary>
    ''' Aktionen zum Rckgngig machen des Befehls
    ''' </summary>
    Public Overrides Sub Undo()
        ' Instanz des zu lschenden IconImage-Objektes holen
        Dim Img As IconImage = m_Datei.GetImage(m_ImagePath)
        ' Dieses Objekt zur Datei hinzufgen
        m_Datei.Add(Img)
    End Sub
End Class

''' <summary>
''' Erstellt ein neues IconImage-Objekt aus einer Pfad-Vorlage 
''' in einem festgelegten Format und ersetzt ggf. ein vorhandenes 
''' mit gleichem Format
''' </summary>
''' <remarks></remarks>
Public Class CreateNewImageCommand
    Inherits Command

    Private m_Datei As IconDatei            ' Referenz zur Icon-Datei
    Private m_Format As IconImageFormat     ' zu erstellendes IconImageformat
    Private m_TemplateImagePath As String   ' Pfad zu einer Bilddatei
    Private m_ReplacedImagePath As String   ' falls vorhanden Temporr-Pfad des zu ersetzenden IconImage-Objekts
    Private m_NewImagePath As String        ' Temporr-Pfad des neuen IconImage-Objekts

    ''' <summary>
    ''' Konstruktor
    ''' </summary>
    ''' <param name="Datei">Referenz zur Icon-Datei</param>
    ''' <param name="ReplacedImage">falls vorhanden, zu ersetzendes IconImage-Objekt</param>
    ''' <param name="NewFomat">zu erstellendes IconImageformat</param>
    ''' <param name="TemplateImagePath">Pfad zu einer Bilddatei</param>
    Public Sub New(ByVal Datei As IconDatei, ByVal ReplacedImage As IconImage, ByVal NewFomat As IconImageFormat, ByVal TemplateImagePath As String)
        m_Datei = Datei
        If ReplacedImage Is Nothing Then
            m_ReplacedImagePath = ""
        Else
            m_ReplacedImagePath = ReplacedImage.TempName
        End If
        m_Format = NewFomat
        m_TemplateImagePath = TemplateImagePath
    End Sub

    ''' <summary>
    ''' Aktionen bei erstmaligem Ausfhren des Commands
    ''' </summary>
    Public Overrides Sub RunFirst()
        ' wenn ein IconImage zu ersetzen ist, dies zuerst aus der Datei entfernen
        If m_ReplacedImagePath <> "" Then
            Dim ImageToReplace As IconImage = m_Datei.GetImage(m_ReplacedImagePath)
            m_Datei.Remove(ImageToReplace)
        End If
        ' neues IconImage erstellen und dieses der Datei hinzufgen
        Dim ImportImage As Image = Image.FromFile(m_TemplateImagePath)
        m_NewImagePath = m_Datei.Add(m_Format, ImportImage).TempName
    End Sub

    ''' <summary>
    ''' Bezeichnung des Befehls, der in der History-Liste angezeigt wird
    ''' </summary>
    Public Overrides ReadOnly Property Name() As String
        Get
            Return "Neues Bild anlegen"
        End Get
    End Property

    ''' <summary>
    ''' Aktionen zum Wiederherstellen
    ''' </summary>
    Public Overrides Sub Redo()
        If m_ReplacedImagePath <> "" Then
            Dim ImageToReplace As IconImage = m_Datei.GetImage(m_ReplacedImagePath)
            m_Datei.Remove(ImageToReplace)
        End If
        Dim NewImage As IconImage = m_Datei.GetImage(m_NewImagePath)
        m_Datei.Add(NewImage)
    End Sub

    ''' <summary>
    ''' Aktionen zum Rckgngig machen des Befehls
    ''' </summary>
    Public Overrides Sub Undo()
        Dim NewImage As IconImage = m_Datei.GetImage(m_NewImagePath)
        m_Datei.Remove(NewImage)
        If m_ReplacedImagePath <> "" Then
            Dim ImageToReplace As IconImage = m_Datei.GetImage(m_ReplacedImagePath)
            m_Datei.Add(ImageToReplace)
        End If
    End Sub
End Class

''' <summary>
''' Erstellt ein IconImage in gewnschtem Format aus einem anderen bereits vorhandenen IconImage 
''' und ersetzt wenn notwendig ein vorhandenes IconImage im selben Format
''' </summary>
Public Class CreateImageCloneCommand
    Inherits Command
    ' Erstellt ein neues Bild aus einem IconImage

    Private m_Datei As IconDatei ' Referenz zur Icon-Datei
    Private m_Format As IconImageFormat

    Private m_TemplateImagePath As String
    Private m_ReplacedImagePath As String
    Private m_NewImagePath As String

    ''' <summary>
    ''' Konstruktor
    ''' </summary>
    ''' <param name="Datei">Referenz zur Icon-Datei</param>
    ''' <param name="ReplacedImage">falls vorhanden, zu ersetzendes IconImage</param>
    ''' <param name="NewFomat">neues IconImageFormat</param>
    ''' <param name="TemplateImage">IconImage, dessen Bildinformationen verwendet werden</param>
    Public Sub New(ByVal Datei As IconDatei, ByVal ReplacedImage As IconImage, ByVal NewFomat As IconImageFormat, ByVal TemplateImage As IconImage)
        m_Datei = Datei
        If ReplacedImage Is Nothing Then
            m_ReplacedImagePath = ""
        Else
            m_ReplacedImagePath = ReplacedImage.TempName
        End If
        m_Format = NewFomat
        m_TemplateImagePath = TemplateImage.TempName
    End Sub

    ''' <summary>
    ''' Aktionen bei erstmaligem Ausfhren des Commands
    ''' </summary>
    Public Overrides Sub RunFirst()
        If m_ReplacedImagePath <> "" Then
            Dim ImageToReplace As IconImage = m_Datei.GetImage(m_ReplacedImagePath)
            m_Datei.Remove(ImageToReplace)
        End If
        Dim TempImage As IconImage = m_Datei.GetImage(m_TemplateImagePath)
        m_NewImagePath = m_Datei.Add(m_Format, TempImage.Bitmap).TempName
    End Sub

    ''' <summary>
    ''' Bezeichnung des Befehls, der in der History-Liste angezeigt wird
    ''' </summary>
    Public Overrides ReadOnly Property Name() As String
        Get
            Return "Neues Bild anlegen"
        End Get
    End Property

    ''' <summary>
    ''' Aktionen zum Wiederherstellen
    ''' </summary>
    Public Overrides Sub Redo()
        If m_ReplacedImagePath <> "" Then
            Dim ImageToReplace As IconImage = m_Datei.GetImage(m_ReplacedImagePath)
            m_Datei.Remove(ImageToReplace)
        End If
        Dim NewImage As IconImage = m_Datei.GetImage(m_NewImagePath)
        m_Datei.Add(NewImage)
    End Sub

    ''' <summary>
    ''' Aktionen zum Rckgngig machen des Befehls
    ''' </summary>
    Public Overrides Sub Undo()
        Dim NewImage As IconImage = m_Datei.GetImage(m_NewImagePath)
        m_Datei.Remove(NewImage)
        If m_ReplacedImagePath <> "" Then
            Dim ImageToReplace As IconImage = m_Datei.GetImage(m_ReplacedImagePath)
            m_Datei.Add(ImageToReplace)
        End If
    End Sub
End Class

''' <summary>
''' Erzeugt IconImages in bevorzugtem Format
''' </summary>
Public Class CreateAutoIconCommand
    Inherits Command
    Private m_Datei As IconDatei ' Referenz zur Icon-Datei
    Private m_ImagePath As String
    Private m_RemovedImageNames As List(Of String)
    Private m_AddedImageNames As List(Of String)
    ''' <summary>
    ''' Konstruktor
    ''' </summary>
    ''' <param name="Datei">Referenz zur Icon-Datei</param>
    ''' <param name="ImagePath"></param>
    ''' <remarks></remarks>
    Public Sub New(ByVal Datei As IconDatei, ByVal ImagePath As String)
        m_Datei = Datei
        m_ImagePath = ImagePath
        m_RemovedImageNames = New List(Of String)
        m_AddedImageNames = New List(Of String)
    End Sub

    ''' <summary>
    ''' Bezeichnung des Befehls, der in der History-Liste angezeigt wird
    ''' </summary>
    Public Overrides ReadOnly Property Name() As String
        Get
            Return "Auto-Icon anlegen"
        End Get
    End Property

    ''' <summary>
    ''' Aktionen bei erstmaligem Ausfhren des Commands
    ''' </summary>
    Public Overrides Sub RunFirst()
        Dim Info As IconDatei.AutoIconInfo = m_Datei.CreateAutoIcon(m_ImagePath)
        m_RemovedImageNames.AddRange(Info.RemovedImages.ToArray)
        m_AddedImageNames.AddRange(Info.CreatedImages.ToArray)
    End Sub

    ''' <summary>
    ''' Aktionen zum Wiederherstellen
    ''' </summary>
    Public Overrides Sub Redo()
        For Each txt As String In m_RemovedImageNames
            Dim ImageToRemove As IconImage = m_Datei.GetImage(txt)
            m_Datei.Remove(ImageToRemove)
        Next
        For Each txt As String In m_AddedImageNames
            Dim ImageToAdd As IconImage = m_Datei.GetImage(txt)
            m_Datei.Add(ImageToAdd)
        Next
    End Sub

    ''' <summary>
    ''' Aktionen zum Rckgngig machen des Befehls
    ''' </summary>
    Public Overrides Sub Undo()
        For Each txt As String In m_AddedImageNames
            Dim ImageToRemove As IconImage = m_Datei.GetImage(txt)
            m_Datei.Remove(ImageToRemove)
        Next
        For Each txt As String In m_RemovedImageNames
            Dim ImageToAdd As IconImage = m_Datei.GetImage(txt)
            m_Datei.Add(ImageToAdd)
        Next
    End Sub
End Class