'  Copyright (c) 2006, Gustavo Franco
'  Email:  gustavo_franco@hotmail.com
'  All rights reserved.

'  Redistribution and use in source and binary forms, with or without modification, 
'  are permitted provided that the following conditions are met:

'  Redistributions of source code must retain the above copyright notice, 
'  this list of conditions and the following disclaimer. 
'  Redistributions in binary form must reproduce the above copyright notice, 
'  this list of conditions and the following disclaimer in the documentation 
'  and/or other materials provided with the distribution. 

'  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
'  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
'  REMAINS UNCHANGED.

Imports System.IO
Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices
''' <summary>
''' Ein Bild innerhalb des Icons im Bitmap-Format
''' </summary>
Public Class BitmapImage
    Inherits IconImage

#Region " Private Felder "
    ' falls notwendig, Farbpalette
    Friend m_Colors() As Color
    ' Pixelwerte fr Farben im Bitmap
    Protected m_XOR() As Byte
    ' Array mit Transparenz-Werten fr jeden Pixel
    Protected m_AND() As Byte
#End Region

#Region " Konstruktoren "
    ''' <summary>
    ''' Kostruktor zum Erstellen einer BitmapImage-Instanz mit Hilfe einer fremden Bitmap als Vorlage
    ''' </summary>
    ''' <param name="Width">gewnschte Kantenlnge</param>
    ''' <param name="BitsPerPixel">gewnschte Farbtiefe</param>
    ''' <param name="BaseImage">Bitmap-Vorlage</param>
    ''' <param name="GifBackColor">Hintergrundfarbe fr Transparenz-Reduktion</param>
    Public Sub New(ByVal Width As Integer, ByVal BitsPerPixel As Integer, ByVal BaseImage As Bitmap, ByVal GifBackColor As Color)
        ' die bergebenen Parameter sind wesentliche Eigenschaften der IconImage-Basisklasse
        MyBase.Width = Width
        MyBase.Height = Width
        MyBase.BitsPerPixel = BitsPerPixel
        MyBase.IsPNG = False


        ' Zum Erstellen neuer eigener Images mit dem IconCreator sind nur diese drei Formate notwendig
        ' (und mglich)
        Dim Fmt As Drawing.Imaging.PixelFormat
        Select Case BitsPerPixel
            Case 32
                Fmt = Imaging.PixelFormat.Format32bppArgb
            Case 8
                Fmt = Imaging.PixelFormat.Format8bppIndexed
            Case 4
                Fmt = Imaging.PixelFormat.Format4bppIndexed
        End Select

        ' Nachschlage-Dictionary fr indizierte Farben
        ' (Enthlt jeweils eine Farbe und den dazu gehrenden Index aus der Farbtabelle)
        Dim ColorIndizes As Dictionary(Of Color, Byte) = Nothing
        If BitsPerPixel <= 8 Then
            'Wenn indizierte Farben verwendet werden, Farbtabelle mit Windows-Standard-Farben auffllen
            Using Tmp As New Bitmap(16, 16, Fmt)
                ReDim m_Colors(Tmp.Palette.Entries.Length - 1)
                For I As Integer = 0 To Tmp.Palette.Entries.Length - 1
                    m_Colors(I) = Tmp.Palette.Entries(I)
                Next
            End Using

            'Erstauffllung des Nachschlage-Dictionary mit den vorhandenen Standard-Farben und dazu gehrendem Index
            ColorIndizes = New Dictionary(Of Color, Byte)
            Dim Zhl As Integer = 0
            For Each Col As Color In m_Colors
                If Not ColorIndizes.ContainsKey(Col) Then
                    ColorIndizes.Add(Col, Zhl)
                End If
                Zhl += 1
            Next
        Else
            ' Auch wenn keine Farbtabelle verwendet wird, das Colors-Array mit 0 Elementen initialisieren,
            ' denn nachher beim Speichern wird die Lnge dieses Arrays abgefragt 
            ' (wenn m_Colors=Nothing, gbe es dann einen Fehler)
            ReDim m_Colors(-1)
        End If



        'Eine 32-Bit-Bitmap, auf die das Vorlagen-Bild gezeichnet wird 
        Using BMP As New Bitmap(Width, Width, Drawing.Imaging.PixelFormat.Format32bppArgb)
            If BaseImage IsNot Nothing Then
                Using g As Graphics = Graphics.FromImage(BMP)
                    g.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBilinear
                    g.DrawImage(BaseImage, New Rectangle(0, 0, Width, Width), 0, 0, BaseImage.Width, BaseImage.Height, GraphicsUnit.Pixel)
                End Using
            End If


            'RotateFlip ist erforderlich, weil die Datenstruktur der XOR- und AND-Daten dies erfordert
            BMP.RotateFlip(RotateFlipType.RotateNoneFlipY)
            'Ein BitmapData-Objekt fr die 32BPP Bitmap erstellen
            Dim BMPData As BitmapData = BMP.LockBits(New Rectangle(0, 0, Width, Width), ImageLockMode.ReadOnly, Drawing.Imaging.PixelFormat.Format32bppArgb)
            Dim ptr As IntPtr = BMPData.Scan0

            'Das Array rgbValues wird mit den ARGB-Werten fr der 32Bit-Bitmap befllt 
            Dim rgbValues() As Byte = Nothing

            'Das Array GrayBitmapValues soll ARGB-Werte fr eine zunchst mit GifBackColor gefllte 32Bit-Bitmap enthalten, 
            'auf die dann das 32Bit-Bitmap mit transparenten Farben gezeichnet wird
            Dim GrayBitmapValues() As Byte = Nothing

            'If m_Format.PixelFormat <> Imaging.PixelFormat.Format32bppArgb Then
            Dim bytes As Integer = Width * Width * 4
            ReDim rgbValues(bytes - 1)
            ' RGB - Werte in ein Array kopieren
            System.Runtime.InteropServices.Marshal.Copy(ptr, rgbValues, 0, bytes)

            'Dieselbe Prozedur wie fr die RGB32Bit-Bitmap. Nur dass diese Bitmap zuerst mit GifBackColor 
            '(als Hintergrundfarbe) gefllt wird
            Using GrayBitmap As New Bitmap(Width, Width, Drawing.Imaging.PixelFormat.Format32bppArgb)
                If BaseImage IsNot Nothing Then
                    Using gGray As Graphics = Graphics.FromImage(GrayBitmap)
                        gGray.Clear(GifBackColor)
                        gGray.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBilinear
                        gGray.DrawImage(BaseImage, New Rectangle(0, 0, Width, Width), 0, 0, BaseImage.Width, BaseImage.Height, GraphicsUnit.Pixel)
                    End Using
                End If

                GrayBitmap.RotateFlip(RotateFlipType.RotateNoneFlipY)

                Dim GrayBMPData As BitmapData = GrayBitmap.LockBits(New Rectangle(0, 0, Width, Width), ImageLockMode.ReadOnly, Drawing.Imaging.PixelFormat.Format32bppArgb)
                Dim Grayptr As IntPtr = GrayBMPData.Scan0

                ReDim GrayBitmapValues(bytes - 1)
                ' RGB - Werte in ein Array kopieren
                System.Runtime.InteropServices.Marshal.Copy(Grayptr, GrayBitmapValues, 0, bytes)


                GrayBitmap.UnlockBits(GrayBMPData)
            End Using


            'XOR-Daten - enthlt Farbwerte
            'Die XOR-Breite (XorStride) ist grer als die reale Breite der Bitmap
            'muss eine Potenz von 2 sein
            Dim XorStride As Integer
            Select Case BitsPerPixel
                Case 32
                    XorStride = Width * 4
                Case 24
                    XorStride = ((Width * 3) + 3) And Not 3
                Case 8
                    XorStride = (Width + 3) And Not 3
                Case 4
                    XorStride = ((Width + 7) And Not 7) \ 2
            End Select
            'Xor-Array erstellen
            ReDim m_XOR(XorStride * Width - 1)

            'in den AND-Daten beschreibt jeweils 1 Bit, ob ein Pixel transparent oder nicht ist 
            Dim Stride1BPP As Integer = ((Width + 31) And Not 31) \ 8
            ReDim m_AND(Stride1BPP * Width - 1)

            'Schritt zum Lesen der 32-Bit-Bitmaps
            Dim Stride32BPP As Integer = Width * 4

            'temporre Positionen fr den Schleifendurchlauf in Folgenden
            Dim PosXor, PosAnd, Pos32 As Integer

            Select Case BitsPerPixel
                Case 32 ' Format32bppArgb
                    'fr 32 Bit pro Pixel:
                    'Die 32Bit-Bitmap-Daten werden hier nur in das Xor-Array kopiert
                    Marshal.Copy(ptr, m_XOR, 0, m_XOR.Length)

                    'nun feststellen, welche Pixel im 32Bit-Bitmap transparent sind
                    For y As Integer = 0 To BMPData.Height - 1
                        PosXor = Stride32BPP * y
                        PosAnd = Stride1BPP * y
                        For x As Integer = 0 To BMPData.Width - 1
                            If m_XOR(PosXor + 3) = 0 Then
                                'Transparenter Pixel
                                Dim Pos1 As Integer = (x >> 3) + PosAnd
                                'zugehriges Pixel-Bit im AND-Array auf True setzen
                                m_AND(Pos1) = m_AND(Pos1) Or CByte((128 >> (x And 7)))
                            End If
                            PosXor += 4
                        Next
                    Next
                Case 24 ' Format24bppRgb
                    ' dies ist eigentlich ein verwaister Eintrag
                    ' 24BPP gibts im IconCreator nicht als Format zum Neuerstellen von Icon-Images
                    'fr 24 Bit pro Pixel:
                    For y As Integer = 0 To BMPData.Height - 1
                        PosXor = XorStride * y
                        Pos32 = Stride32BPP * y
                        PosAnd = Stride1BPP * y
                        For x As Integer = 0 To Width - 1
                            If rgbValues(Pos32 + 3) < 128 Then
                                'Transparenter Pixel
                                Dim Pos1 As Integer = (x >> 3) + PosAnd
                                'zugehriges Pixel-Bit im AND-Array auf True setzen
                                m_AND(Pos1) = m_AND(Pos1) Or CByte((128 >> (x And 7)))
                            Else
                                'Nichttransparenter Pixel
                                'Bei 24BPP gibt es fr jeden Pixel nur Red, Green, Blue
                                m_XOR(PosXor) = GrayBitmapValues(Pos32)
                                m_XOR(PosXor + 1) = GrayBitmapValues(Pos32 + 1)
                                m_XOR(PosXor + 2) = GrayBitmapValues(Pos32 + 2)
                            End If

                            PosXor += 3
                            Pos32 += 4
                        Next
                    Next
                Case 8 ' Format8bppIndexed
                    'fr 8 Bit pro Pixel (indizierte Farben):
                    For y As Integer = 0 To BMPData.Height - 1
                        PosXor = XorStride * y
                        Pos32 = Stride32BPP * y
                        PosAnd = Stride1BPP * y
                        For x As Integer = 0 To Width - 1
                            If rgbValues(Pos32 + 3) < 128 Then
                                'Transparenter Pixel
                                Dim Pos1 As Integer = (x >> 3) + PosAnd
                                'zugehriges Pixel-Bit im AND-Array auf True setzen
                                m_AND(Pos1) = m_AND(Pos1) Or CByte((128 >> (x And 7)))
                            Else
                                'Nichttransparenter Pixel
                                'Fr die dazu gehrende Farbe den passenden Farbindex aus den 
                                'Windows-Standardfarben suchen und diesen Wert hier einfgen
                                'bei 8BPP steht fr den Farbindex 1Byte zur Verfgung
                                m_XOR(PosXor) = GetNearColorIndex(Color.FromArgb(GrayBitmapValues(Pos32 + 2), GrayBitmapValues(Pos32 + 1), GrayBitmapValues(Pos32)), ColorIndizes)
                            End If
                            PosXor += 1
                            Pos32 += 4
                        Next
                    Next
                Case 4 ' Format4bppIndexed
                    'fr 4 Bit pro Pixel (indizierte Farben):
                    For y As Integer = 0 To BMPData.Height - 1
                        PosXor = XorStride * y
                        Pos32 = Stride32BPP * y
                        PosAnd = Stride1BPP * y
                        For x As Integer = 0 To Width - 1
                            If rgbValues(Pos32 + 3) < 128 Then
                                'Transparenter Pixel
                                Dim Pos1 As Integer = (x >> 3) + PosAnd
                                'zugehriges Pixel-Bit im AND-Array auf True setzen
                                m_AND(Pos1) = m_AND(Pos1) Or CByte((128 >> (x And 7)))
                            Else
                                'Nichttransparenter Pixel
                                'hier stehen fr jede Farbangabe nur 4 Bit zur Verfgung, d.h.
                                'zwei Farbindizes teilen sich ein Byte
                                Dim xPosXor As Integer = (x >> 1) + PosXor
                                If (x And 1) = 0 Then
                                    m_XOR(xPosXor) = m_XOR(xPosXor) Or (GetNearColorIndex(Color.FromArgb(GrayBitmapValues(Pos32 + 2), GrayBitmapValues(Pos32 + 1), GrayBitmapValues(Pos32)), ColorIndizes) << 4)
                                Else
                                    m_XOR(xPosXor) = m_XOR(xPosXor) Or GetNearColorIndex(Color.FromArgb(GrayBitmapValues(Pos32 + 2), GrayBitmapValues(Pos32 + 1), GrayBitmapValues(Pos32)), ColorIndizes)
                                End If
                            End If
                            Pos32 += 4
                        Next
                    Next
            End Select

            'BitmapData frei geben
            BMP.UnlockBits(BMPData)
            'Bitmap frei geben
        End Using
        'Bitmap erstellen
        ' Als Ergebnis haben wir ein ByteArray mit Farbdaten (XOR), ein ByteAray mit Transparenz-Daten (AND)
        ' Breite, Hhe und Farbtiefe

        ' Nun kann die Bitmap zum Anzeigen des Inhalts erstellt werden:
        Dim ms As New MemoryStream
        Dim writer As New BinaryWriter(ms)

        ' IconHeader schreiben
        Dim Head As New IconHeader(1)
        Head.Write(writer)
        ' IconDirectory schreiben
        Dim Directory As New IconDirectory(Width, Height, m_Colors.Length, 40 + 4 * m_Colors.Length + m_XOR.Length + m_AND.Length, 6 + 16, BitsPerPixel)
        Directory.Write(writer)
        ' Bilddaten schreiben
        Dim Header As New BitmapInfoHeader(Width, Width * 2, m_Colors.Length, BitsPerPixel, m_XOR.Length)
        Header.Write(writer)

        ' Farbpalette schreiben
        For i As Integer = 0 To m_Colors.Length - 1
            ' Image Palette
            writer.Write(m_Colors(i).B)
            writer.Write(m_Colors(i).G)
            writer.Write(m_Colors(i).R)
            writer.Write(CByte(0))
        Next

        ' XOR Informationen schreiben
        ms.Write(m_XOR, 0, m_XOR.Length)

        ' AND Informationen schreiben
        ms.Write(m_AND, 0, m_AND.Length)

        ms.Position = 0

        ' Die eben in den Stream geschriebenen Daten sind eigentlich ein Icon mit nur einem Image
        ' jetzt diesen Icon mit Hilfe des Strams erstellen und Bitmap mit gewnschten Abmessungen 
        ' abrufen
        Dim ico As New Icon(ms, Width, Height)
        ms.Dispose()
        m_BMP = ico.ToBitmap

        ' als letztes die notwendige Daten-Struktur fr das Speichern dieses neuen IconImage erstellen
        ms = New MemoryStream
        writer = New BinaryWriter(ms)
        Header.Write(writer)

        ' Farbpalette
        For i As Integer = 0 To m_Colors.Length - 1
            ' Image Palette
            writer.Write(m_Colors(i).B)
            writer.Write(m_Colors(i).G)
            writer.Write(m_Colors(i).R)
            writer.Write(CByte(0))
        Next

        ' XOR Daten
        ms.Write(m_XOR, 0, m_XOR.Length)

        ' AND Daten
        ms.Write(m_AND, 0, m_AND.Length)

        Dim buffer() As Byte = ms.GetBuffer()
        m_ImageData = buffer

        Dim ColorCount As Integer = 0
        If BitsPerPixel = 4 Then
            ColorCount = 16
        End If

        m_Directory = New IconDirectory(m_BMP.Width, m_BMP.Height, ColorCount, m_ImageData.Length, 0, BitsPerPixel)
    End Sub

    ''' <summary>
    ''' Konstruktor mit Daten aus einem Stream 
    ''' </summary>
    Public Sub New(ByVal Directory As IconDirectory, ByVal ImageData As Byte())
        ' Original-Directory-Daten und Image-Daten gleich in einem Feld speichern
        ' sie werden beim Speichern im wesentlichen genau so verwandt, wie sie gelesen wurden
        m_Directory = Directory
        m_ImageData = ImageData

        ' Also dient die erstellte Bitmap auch nur dem Anzeigen und nicht dem Speichern

        ' aus dem BitmapInfoHeader, der im ImageData-Byte-Array enthalten ist,
        ' knnen gleich die Abmessungen und Farbtiefe der aktuellen Bitmap ermittelt werden
        Dim header As New BitmapInfoHeader(ImageData)

        Me.Width = header.Width
        Me.Height = header.Height \ 2
        Me.BitsPerPixel = header.BitCount
        Me.IsPNG = False




        ' Anzahl Farben in der Farbpalette ermitteln
        If BitsPerPixel <= 8 Then
            ' bei mit Sicherheit indizierten Farben (abhngig von BitCount)
            ' hat die Palette eine fest definierte Gre
            ReDim m_Colors((1 << BitsPerPixel) - 1)
        ElseIf header.ClrUsed > 0 Then
            ' in anderen Fllen kann trotzdem eine Farbpalette vorhanden sein
            ' Deren Grsse hngt dann von ClrUsed ab
            ReDim m_Colors(header.ClrUsed - 1)
        Else
            ' ansonsten enthlt die Palette keine Farben
            ReDim m_Colors(-1)
        End If

        ' in den Bitmap-Daten knnen eventuell weniger Palettenfarben als 
        ' in der Farbpalette mglich, enthalten sein
        Dim NumColorsInImageData As Integer = 0

        ' Wenn eine Angabe zu ClrUsed vorhanden ist, entspricht diese Angabe 
        ' der Anzahl im Byte-Array vorhandenen Farben
        If header.ClrUsed > 0 Then
            NumColorsInImageData = header.ClrUsed
        ElseIf BitsPerPixel <= 8 Then
            ' ansonsten wird, wenn hier indizierte Farben verwendet werden,
            ' davon ausgegangen, dass Farben fr die gesamte Palette vorhanden sind
            NumColorsInImageData = 1 << BitsPerPixel
        End If


        Dim AcIndex As Integer = 40
        ' Farbpalette mit verwendeten Farben fllen
        For I As Integer = 0 To NumColorsInImageData - 1
            m_Colors(I) = Color.FromArgb(ImageData(AcIndex + 2), ImageData(AcIndex + 1), ImageData(AcIndex))
            AcIndex += 4
        Next

        ' Der XOR-Abschnitt enhlt Farbdaten fr die Bitmap. Sie sind hier ebenso als Byte-Folge
        ' gespeichert, wie das in einer realen Bitmap mit gleicher Farbtiefe vorzufinden wre
        Dim stride As Integer = ((Width * BitsPerPixel + 31) And Not 31) >> 3
        ReDim m_XOR(stride * Height - 1)
        Array.Copy(ImageData, AcIndex, m_XOR, 0, m_XOR.Length)
        AcIndex += m_XOR.Length

        ' Der AND-Abschnitt enthlt bitweise fr jeden Pixel der Bitmap eine Information
        ' ob dieser Pixel im Icon transparent erscheint oder nicht
        stride = ((Width + 31) And Not 31) >> 3
        ReDim m_AND(stride * Height - 1)
        Array.Copy(ImageData, AcIndex, m_AND, 0, m_AND.Length)

        ' jetzt sind alle Daten gelesen. Zum Speichern wurden sie schon vorin in einer lokalen Variablen
        ' untergebracht. Nun kann die Bitmap zum Anzeigen des IconImages erstellt werden.
        ' Eine einfache Bitmap mit XOR-Daten reicht hier nicht - es knnte ja sein, dass diese
        ' Bitmap gar keine Transparenz-Informationen enthlt. Also geht man folgenen Weg: 
        ' Man erzeugt ein Icon mit nur einem einzelnen IconImage - eben diesem hier.
        ' Aus dem Icon wird dann eine 32-Bit-Bitmap mit transparenten Pixeln erzeugt.

        ' neuen MemoryStream anlegen
        Dim ms As New MemoryStream
        Dim writer As New BinaryWriter(ms)

        ' IconHeader schreiben
        Dim Head As New IconHeader(1)
        Head.Write(writer)
        ' IconDirectory schreiben
        Dim IcnDirectory As New IconDirectory(Width, Height, header.ClrUsed, ImageData.Length, 6 + 16, BitsPerPixel)
        IcnDirectory.Write(writer)
        ' Bilddaten schreiben
        writer.Write(ImageData)

        ms.Position = 0

        ' Icon aus Stream erzeugen
        Dim ico As New Icon(ms, Width, Height)
        ' Stream frei geben
        ms.Dispose()
        ' Bitmap aus Icon erstellen
        m_BMP = ico.ToBitmap
        ' Icon frei geben
        ico.Dispose()
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' liefert den geeigneten Farbindex aus einer Farbtabelle fr eine gesuchte Pixel-Farbe 
    ''' </summary>
    ''' <param name="SCol">gesuchte Farbe</param>
    ''' <param name="ColorIndizes">Temporr verwendetes Dictionary mit Farben und zugehrigem Index aus der Farbtabelle</param>
    ''' <returns>Index aus der Farbtabelle</returns>
    Private Function GetNearColorIndex(ByVal SCol As Color, ByVal ColorIndizes As Dictionary(Of Color, Byte)) As Byte

        If ColorIndizes.ContainsKey(SCol) Then
            ' wenn die gesuchte Farbe bereits im Dictionary enthalten ist,
            ' dazu gehrenden Farbindex zurck geben
            Return ColorIndizes(SCol)
        Else
            ' wenn die gesuchte Farbe noch nicht im Dictionary enthalten ist, ...

            ' Variable fr nachst liegende Farbe
            Dim NearestColor As Color = Nothing
            ' deren zugeordneter Index
            Dim Idx As Byte

            ' Distanz zur nchstliegenden Farbe - erhlt erst mal den maximal mglichen Wert
            Dim MinDistance As Integer = Integer.MaxValue

            For Each Col As Color In m_Colors
                ' alle Farben in der Farbpalette durchgehen und deren Distanz zur gesuchten Farbe bestimmen
                Dim redDistance As Integer = CInt(Col.R) - SCol.R
                Dim greenDistance As Integer = CInt(Col.G) - SCol.G
                Dim blueDistance As Integer = CInt(Col.B) - SCol.B

                Dim Distance As Integer = (redDistance * redDistance) + (greenDistance * greenDistance) + (blueDistance * blueDistance)

                ' Wenn die ermittelte Farbdistanz geringer ist als die zuletzt bekannte
                If Distance < MinDistance Then
                    ' wird die aktuelle Farbe zur nchstliegenden,
                    NearestColor = Col
                    MinDistance = Distance

                    ' Wenn der RGB-Wert exakt getroffen wurde, Farbe zum Dictionary hinzufgen 
                    ' und Index zurckgeben
                    If 0 = Distance Then
                        Idx = ColorIndizes(NearestColor)
                        ColorIndizes.Add(SCol, Idx)
                        Return Idx
                    End If
                End If
            Next

            ' die gefundene nchstliegende Farbe mit Index zum Dictionary
            ' hinzufgen und Index zurck geben

            Idx = ColorIndizes(NearestColor)
            ColorIndizes.Add(SCol, Idx)
            Return Idx

        End If
    End Function

    ''' <summary>
    ''' Dummy-Methode zum Demonstrieren, wie man aus den XOR-Daten eine (ggf. indizierte) Bitmap erstellt
    ''' </summary>
    ''' <param name="FileName">Pfad, unter dem die Bitmap gespeichert werden soll</param>
    ''' <remarks>Diese Methode wird im IconCreator nicht verwendet</remarks>
    Public Sub CreateExtraBitmap(ByVal FileName As String)

        ' BMP file header
        ' Offset#    Size    Purpose 
        ' 0          2       the magic number used to identify the BMP file: 0x42 0x4D (ASCII code points for B and M) 
        ' 2          4       the size of the BMP file in bytes 
        ' 6          2       reserved; actual value depends on the application that creates the image 
        ' 8          2       reserved; actual value depends on the application that creates the image 
        ' 10         4       the offset, i.e. starting address, of the byte where the bitmap data can be found. 

        ' Bitmap information (DIB header)

        ' Color palette

        ' neuen FileStream anlegen
        Dim Stream As New IO.FileStream(FileName, FileMode.Create, FileAccess.ReadWrite)

        ' the size of the BMP file in bytes 
        ' = BMP file header 
        ' + Bitmap information
        ' + Color palette
        ' + Bitmap Data

        Dim Bitmapsize As Integer = 14 + 40 + 4 * m_Colors.Length + m_XOR.Length
        Dim BitmapDataOffset As Integer = 14

        Dim TmpShort As Short = 0

        Try


            Stream.Position = 0
            Dim writer As New BinaryWriter(Stream)
            ' BMP Datei Header schreiben
            writer.Write(CByte(66))
            writer.Write(CByte(77))
            writer.Write(Bitmapsize)

            writer.Write(TmpShort)
            writer.Write(TmpShort)
            writer.Write(BitmapDataOffset)

            ' Bitmap information header (DIB header) schreiben

            Dim TmpHeader As New BitmapInfoHeader(m_BMP.Width, m_BMP.Height, m_Colors.Length, BitsPerPixel, m_XOR.Length)
            ' Bitmap Header

            TmpHeader.Write(writer)

            ' Farbpalette schreiben
            For i As Integer = 0 To m_Colors.Length - 1
                writer.Write(m_Colors(i).B)
                writer.Write(m_Colors(i).G)
                writer.Write(m_Colors(i).R)
                writer.Write(CByte(0))
            Next


            ' XOR-Daten schreiben
            Stream.Write(m_XOR, 0, m_XOR.Length)


        Catch ex As Exception
            MsgBox(ex.Message)
        Finally

            If Stream IsNot Nothing Then
                Stream.Close()
            End If
        End Try

    End Sub
#End Region

End Class
