''' <summary>
''' liefert indizierte Farben fr eine 32 bit Bitmap
''' </summary>
Public Class ColorIndexer

#Region " Private Felder "

    ' Index der transparenten Farbe (kann auch -1 sein)
    Private m_TransparentColorIndex As Integer = -1
    ' Farbindizes
    Private m_Indizes() As Byte

#End Region

#Region " Properties "
    ''' <summary>
    ''' Farbindizes
    ''' </summary>
    ReadOnly Property Indizes() As Byte()
        Get
            Return m_Indizes
        End Get
    End Property
    ''' <summary>
    ''' Index der transparenten Farbe in der Farbpalette (kann -1 sein, wenn keine transparente Farbe enthalten)
    ''' </summary>
    ReadOnly Property TransparentColorIndex() As Integer
        Get
            Return m_TransparentColorIndex
        End Get
    End Property
#End Region

#Region " Konstruktoren "
    ''' <summary>
    ''' Default Konstruktor
    ''' </summary>
    ''' <param name="Colors">Array mit Pixelfarben</param>
    ''' <param name="PaletteColors">Palettenfarben</param>
    Public Sub New(ByVal Colors() As Color, ByVal PaletteColors() As Color)
        ' Temporres Array mit Farb-Indizes fr jeden Pixel als Integer 
        ' (hier knnte -1 fr transparente Pixel enthalten sein)
        Dim TempIndizes(Colors.Length - 1) As Integer
        ' Dictionary mit Farben und zugeordneten Indizes
        Dim ColorDict As New Dictionary(Of Color, Byte)
        ' Zweites Dictionary, das die Hufigkeit der indizierten Farben zhlt
        Dim ColorCounter As New Dictionary(Of Color, Integer)

        ' alle verwendeten Palettenfarben ind den Dictionarys bekannt machen
        For I As Integer = 0 To PaletteColors.Length - 1
            If Not ColorDict.ContainsKey(PaletteColors(I)) Then
                ' Palettenfarbe und -index zum Farb-Dictionary hinzufgen
                ColorDict.Add(PaletteColors(I), I)
                ' Palettenfarbe (und zunchst 0) zum Farben-Zhl-Dictionary hinzufgen
                ColorCounter.Add(PaletteColors(I), 0)
            End If
        Next

        ' extra Zhler fr transparente Farbe
        Dim TransparentCount As Integer = 0

        ' alle in der 32 Bit-Quelle enthaltenen Pixelfarben durchgehen
        For I As Integer = 0 To Colors.Length - 1
            If Colors(I).A < 128 Then
                ' wenn die Pixelfarbe nicht mal haltransparent ist, wird dieser Pixel transparent
                TempIndizes(I) = -1
                ' Zhler fr transparente Pixel um 1 erhhen
                TransparentCount += 1
            Else
                ' im anderen Fall wird der Pixel nichttransparent und erhlt einen Farbindex
                ' Farbindex mit Hilfe des Dictionarys ermitteln
                Dim FoundIndex As Integer = GetNearColorIndex(Colors(I), PaletteColors, ColorDict)
                ' Zhler fr diese Pixelfarbe um 1 erhhen
                ColorCounter(PaletteColors(FoundIndex)) += 1
                ' dieser Index wird erst einmal in den temporren Indizes gespeichert
                TempIndizes(I) = FoundIndex
            End If
        Next


        If TransparentCount = 0 Then
            ' wenn keine transparenten Farben enthalten sind, braucht die Palette nicht gendert werden
            ReDim m_Indizes(Colors.Length - 1)
            ' (alle temporren Indizes sind > -1)
            For I As Integer = 0 To Colors.Length - 1
                m_Indizes(I) = TempIndizes(I)
            Next
            ' wenn keine transparenten Farben enthalten sind, ist nichts weiter zu tun
            Exit Sub
        End If

        ' es sind transparente Farben enthalten
        ' also wird auf die Indizierung auf die am seltensten gezhlten Farbe verzichtet,
        ' deren Position in der Palette wird Position der neuen transparenten Farbe

        ' eine Variable fr gefunden seltenste Hufigkeit einer Palettenfarbe
        ' erhlt zunchst den maximal mglichen Wert
        Dim SeltensteHaeufigkeit As Integer = Integer.MaxValue
        ' gesuchter Index der seltensten Farbe
        Dim Idx As Integer = 0

        ' alle Palettenfarben durchgehen
        For Each col As Color In PaletteColors
            ' wenn eine Farbe seltener als die zuletzt bekannte gezhlt wurde,
            If ColorCounter(col) < SeltensteHaeufigkeit Then
                ' wird deren Index als seltenster vermerkt
                SeltensteHaeufigkeit = ColorCounter(col)
                ' der seltenste Index wird zum Index der spteren transparenten Farbe
                m_TransparentColorIndex = Idx
            End If
            Idx += 1
        Next


        ' die so ermittelte seltenste Farbe wird durch Transparent ersetzt. 
        ' Temporre neue Farb-Palette (ohne die seltenste Farbe) erstellen
        Dim TmpPalette(PaletteColors.Length - 2) As Color

        ' mit dieser neuen reduzierten Palette werden alle Farbindizes neu bestimmt

        ' Das Farb-Dictionary muss, weil weniger Farben zur Auswahl stehen, neu 
        ' gefllt werden
        ColorDict.Clear()


        ' hier genau wie weiter oben dem Farb-Dictionary und der temporren reduzierten 
        ' Palette die neu zu verwendenden Farben hinzufgen
        For I As Integer = 0 To PaletteColors.Length - 1
            If I <> m_TransparentColorIndex Then
                If I < m_TransparentColorIndex Then
                    TmpPalette(I) = PaletteColors(I)
                ElseIf I > m_TransparentColorIndex Then
                    TmpPalette(I - 1) = PaletteColors(I)
                End If
                If Not ColorDict.ContainsKey(PaletteColors(I)) Then
                    ColorDict.Add(PaletteColors(I), I)
                End If
            End If
        Next


        ReDim m_Indizes(Colors.Length - 1)

        ' nun werden in einer Schleife fr alle Pixelfarben die Indizes neu ermittelt
        For I As Integer = 0 To Colors.Length - 1
            ' weniger als halbtransparente Pixel erhalten den transparenten Index
            If Colors(I).A < 128 Then
                m_Indizes(I) = m_TransparentColorIndex
            Else
                ' alle anderen Pixelfarben erhalten einen passenden freien Farbindex
                m_Indizes(I) = GetNearColorIndex(Colors(I), TmpPalette, ColorDict)
            End If
        Next

    End Sub
#End Region

#Region " Methoden "
    ''' <summary>
    ''' liefert einen Index fr eine gesuchte Farbe aus einer Farbpalette mit Hilfe eines Farben-Dictionarys
    ''' </summary>
    ''' <param name="SCol">gesuchte Farbe</param>
    ''' <param name="PaletteColors">Farbpalette</param>
    ''' <param name="ColorIndizes">Farben-Dictionary</param>
    Private Function GetNearColorIndex(ByVal SCol As Color, ByVal PaletteColors() As Color, ByVal ColorIndizes As Dictionary(Of Color, Byte)) As Byte

        If ColorIndizes.ContainsKey(SCol) Then
            ' Wenn die gesuchte Farbe im Index-Dictionary enthalten ist, wird deren Index zurck gegeben
            Return ColorIndizes(SCol)
        Else
            ' die gesuchte Farbe ist im Dictionary noch nicht enthalten

            ' nchst liegende Farbe
            Dim NearestColor As Color = Nothing
            ' Index der Farbe in der Farbpalette
            Dim Idx As Byte

            ' Minimale Farbdistanz - wird erst mal so gross wie mglich gemacht
            Dim MinDistance As Integer = Integer.MaxValue

            For Each Col As Color In PaletteColors
                ' jetzt alle Farben der Farbpalette auf ihre Distanz zur gesuchten Farbe prfen

                Dim redDistance As Integer = CInt(Col.R) - SCol.R
                Dim greenDistance As Integer = CInt(Col.G) - SCol.G
                Dim blueDistance As Integer = CInt(Col.B) - SCol.B

                ' Gesamt-Distanz der aktuellen Palettenfarbe zur gesuchten Farbe
                Dim Distance As Integer = (redDistance * redDistance) + (greenDistance * greenDistance) + (blueDistance * blueDistance)

                ' wenn diese Distanz kleiner als die zuletzt bekannte Minimal-Distanz ist
                If Distance < MinDistance Then
                    ' wird diese Farbe zur nchstliegenden Farbe
                    NearestColor = Col
                    ' neue Minimaldistanz festhalten
                    MinDistance = Distance

                    ' Wenn der RGB-Wert exakt getroffen wurde, Farbe zum Dictionary hinzufgen 
                    ' und Index zurckgeben
                    If 0 = Distance Then
                        Idx = ColorIndizes(NearestColor)
                        ColorIndizes.Add(SCol, Idx)
                        Return Idx
                    End If
                End If
            Next


            ' an dieser Stelle sind alle Farben untersucht.
            ' Wenigstens eine Farbe ist NearestColor
            ' Deren Index wird dann der gesuchten Farbe zugeordnet, und die gesuchte Farbe fr sptere 
            ' Verwendung mit Index zum Dictionary hinzu gefgt
            ' der gefundene Paletten-Index wird zurck gegeben
            Idx = ColorIndizes(NearestColor)
            ColorIndizes.Add(SCol, Idx)
            Return Idx

        End If
    End Function
#End Region

End Class
