'  Copyright (c) 2006, Gustavo Franco
'  Email:  gustavo_franco@hotmail.com
'  All rights reserved.

'  Redistribution and use in source and binary forms, with or without modification, 
'  are permitted provided that the following conditions are met:

'  Redistributions of source code must retain the above copyright notice, 
'  this list of conditions and the following disclaimer. 
'  Redistributions in binary form must reproduce the above copyright notice, 
'  this list of conditions and the following disclaimer in the documentation 
'  and/or other materials provided with the distribution. 

'  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
'  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
'  REMAINS UNCHANGED.

Imports System.IO
Imports System.Drawing.Imaging
''' <summary>
''' Ein Bild innerhalb des Icons im PNG-Format
''' </summary>
Public Class PNGImage
    Inherits IconImage

#Region " Konstruktoren "
    ''' <summary>
    ''' Kostruktor zum Erstellen einer BitmapImage-Instanz mit Hilfe einer fremden Bitmap als Vorlage
    ''' </summary>
    ''' <param name="Width">gewnschte Kantenlnge</param>
    ''' <param name="BitsPerPixel">gewnschte Farbtiefe</param>
    ''' <param name="BaseImage">Bitmap-Vorlage</param>
    ''' <param name="GifBackColor">Hintergrundfarbe fr Transparenz-Reduktion</param>
    Public Sub New(ByVal Width As Integer, ByVal BitsPerPixel As Integer, ByVal BaseImage As Bitmap, ByVal GifBackColor As Color)
        ' die bergebenen Parameter sind gleichzeitig wesentliche des aktuellen Bildformats
        MyBase.Width = Width
        MyBase.Height = Width
        MyBase.BitsPerPixel = BitsPerPixel
        MyBase.IsPNG = True

        ' Dieser Konstruktor wird nur beim internen Erstellen neuer IconImages 
        ' im IconCreator aufgerufen.
        ' Hier steht also fest: Fr BitsPerPixel gibt es nur 32, 8 oder 4
        ' Die hier erstellte Bitmap wird auch zum Speichern als PNG verwandt
        ' also muss fr indizierte Farben auch eine entsprechend reduzierte 
        ' Bitmap erstellt werden
        MakeBitmap(BaseImage, GifBackColor)

        ' an dieser Stelle ist m_BMP in gewnschter Farbtiefe vorhanden
        ' aus dieser werden die zum spteren Speichern notwendigen Daten erstellt

        Dim ms As New MemoryStream
        ' Bitmap (enthlt gewnschte Farbtiefe) als PNG im Stream speichern
        m_BMP.Save(ms, Drawing.Imaging.ImageFormat.Png)
        Dim buffer() As Byte = ms.GetBuffer()
        ' Die Bytes der als PNG im Stream gespeicherten Bitmap als Bilddaten festhalten
        m_ImageData = buffer

        ' Farbanzahl fr IconDirectory (weicht nur von 0 ab, wenn Farbtiefe = 4 BPP)
        Dim ColorCount As Integer = 0
        If BitsPerPixel = 4 Then
            ColorCount = 16
        End If

        ' IconDirectory zum spteren Speichern erstellen
        m_Directory = New IconDirectory(m_BMP.Width, m_BMP.Height, ColorCount, m_ImageData.Length, 0, BitsPerPixel)

    End Sub

    ''' <summary>
    ''' Konstruktor mit Daten aus einem Stream 
    ''' </summary>
    Public Sub New(ByVal Directory As IconDirectory, ByVal ImageData As Byte())
        ' Original-Directory-Daten und Image-Daten gleich in einem Feld speichern
        ' sie werden beim Speichern im wesentlichen genau so verwandt, wie sie gelesen wurden
        m_Directory = Directory
        m_ImageData = ImageData

        ' eine Bitmap zum Anzeigen des enthaltenen Bildinhalts erstellen
        ' die bergebenden ImageData-Bytes enthalten bereits ein komplettes PNG-Image
        Dim ms As New MemoryStream(ImageData)
        m_BMP = New Bitmap(ms)

        ' Frormat-Eigenschaften (in der Basisklasse enthalten) festlegen
        Me.Width = m_BMP.Width
        Me.Height = m_BMP.Height
        Me.BitsPerPixel = m_Directory.BitCount

        Me.IsPNG = True
    End Sub

#End Region

#Region " Methoden "
    ''' <summary>
    ''' Erstellt eine Bitmap fr das in der Basisklasse bereits enthaltene Bildformat
    ''' mit Hilfe einer Bitmap-Vorlage und einer Hintergrundfarbe fr teilweise transparente Pixel
    ''' </summary>
    ''' <param name="BaseImage">Bitmap-Vorlage</param>
    ''' <param name="GifBackColor">Hintergrundfarbe fr teilweise transparente Pixel</param>
    Private Sub MakeBitmap(ByVal BaseImage As Bitmap, ByVal GifBackColor As Color)

        ' Wenn eine 32-Bit-Bitmap erstellt werden soll
        If BitsPerPixel = 32 Then
            ' Neue 32-Bit-Bitmap erstellen
            m_BMP = New Bitmap(Width, Width, Imaging.PixelFormat.Format32bppArgb)
            ' auf das Graphics-Objekt der neuen 32-Bit-Bitmap die Bitmap-Vorlage zeichnen
            Using g As Graphics = Graphics.FromImage(m_BMP)
                g.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBilinear
                g.DrawImage(BaseImage, New Rectangle(0, 0, Width, Width), 0, 0, BaseImage.Width, BaseImage.Height, GraphicsUnit.Pixel)
            End Using
            Exit Sub
            ' Fr 32BPP ist die Arbeit hier beendet
        End If

        ' Weil teilweise transparente Pixel bei Bedarf einen farbigen Hintergrund erhalten sollen,
        ' brauche ich Daten aus zwei 32-Bit-Bitmaps 
        ' auf die erste Bitmap (TransBMP) wird die Bitmap-Vorlage gezeichnet
        ' diese Bitmap enthlt dann auch halbtransparente Pixel

        ' die zweite Bitmap (GrayBMP) wird erst mit GifBackColor gefllt und darauf
        ' die Bitmap-Vorlage gezeichnet
        ' Wenn GifBackColor also eine nichttransparente Farbe ist, enthlt GrayBMP auch keine 
        ' transparenten Pixel

        Using TransBMP As New Bitmap(Width, Width, Imaging.PixelFormat.Format32bppArgb)
            Using g As Graphics = Graphics.FromImage(TransBMP)
                g.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBilinear
                g.DrawImage(BaseImage, New Rectangle(0, 0, Width, Width), 0, 0, BaseImage.Width, BaseImage.Height, GraphicsUnit.Pixel)
            End Using

            ' Bitmap-Daten fr transparentes Bild
            Dim TransBMPData As BitmapData = TransBMP.LockBits(New Rectangle(0, 0, Width, Width), ImageLockMode.ReadOnly, Drawing.Imaging.PixelFormat.Format32bppArgb)
            Dim ptr As IntPtr = TransBMPData.Scan0

            Dim ByteCount As Integer = Width * Width * 4

            ' Dieses Byte-Array soll Farbwerte fr jeden einzelnen Pixel enthalten
            Dim TranspBytes(ByteCount - 1) As Byte

            ' Das Array TranspBytes() wird mit den ARGB-Werten der 32Bit-Bitmap befllt 
            ' RGB - Werte in das Array kopieren
            System.Runtime.InteropServices.Marshal.Copy(ptr, TranspBytes, 0, ByteCount)
            ' Bitmap-Daten fr transparentes Bild freigeben
            TransBMP.UnlockBits(TransBMPData)

            ' die zweite 32 bit-Bitmap mit GifBackColor-Hintergrund
            Using GrayBMP As New Bitmap(Width, Width, Imaging.PixelFormat.Format32bppArgb)
                Using g As Graphics = Graphics.FromImage(GrayBMP)
                    g.Clear(GifBackColor)
                    g.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBilinear
                    g.DrawImage(BaseImage, New Rectangle(0, 0, Width, Width), 0, 0, BaseImage.Width, BaseImage.Height, GraphicsUnit.Pixel)
                End Using

                ' Bitmap-Daten fr GifBackColor-Bild
                Dim GrayBMPData As BitmapData = GrayBMP.LockBits(New Rectangle(0, 0, Width, Width), ImageLockMode.ReadOnly, Drawing.Imaging.PixelFormat.Format32bppArgb)
                ptr = GrayBMPData.Scan0

                ' Dieses Byte-Array soll Farbwerte fr jeden einzelnen Pixel 
                ' aus dem GifBackColor-Bild enthalten
                Dim GrayBytes(ByteCount - 1) As Byte

                ' Das Array GrayBytes() wird mit den ARGB-Werten der 32Bit-GifBackColor-Bitmap befllt 
                ' RGB - Werte in das Array kopieren
                System.Runtime.InteropServices.Marshal.Copy(ptr, GrayBytes, 0, ByteCount)
                ' Bitmap-Daten fr GifBackColor-Bild freigeben
                GrayBMP.UnlockBits(GrayBMPData)

                ' Array mit Farbwerten fr jeden Pixel, die das neue Bild erhalten soll
                Dim ResultColors(Width * Width - 1) As Color

                ' diese Farbwerte werden aus den vorher erstellten 32-Bit-Bitmaps ermittelt
                Dim Pos32 As Integer
                For I As Integer = 0 To Width * Width - 1
                    Pos32 = I * 4
                    ' zunchst Farbe aus Bitmap mit Transparenz untersuchen
                    Dim TestFarbe As Color = Color.FromArgb(TranspBytes(Pos32 + 3), TranspBytes(Pos32 + 2), TranspBytes(Pos32 + 1), TranspBytes(Pos32))
                    Select Case TestFarbe.A
                        Case 255
                            ' wenn diese Farbe gar nicht transparent ist, wird diese auch zunchst 
                            ' verwendet
                            ResultColors(I) = TestFarbe
                        Case Is < 128
                            ' wenn diese Farbe sehr stark transparent ist, wird der Pixel transparent
                            ResultColors(I) = Color.Transparent
                        Case Else
                            ' wenn diese Farbe nur gering transparent ist, 
                            ' wird die entsprechende nichttransparente Farbe aus den GrayBytes verwendet
                            ResultColors(I) = Color.FromArgb(GrayBytes(Pos32 + 3), GrayBytes(Pos32 + 2), GrayBytes(Pos32 + 1), GrayBytes(Pos32))
                    End Select
                Next

                ' an dieser Stelle habe ich ein Array mit Farben, jede entweder vollstndig transparent 
                ' oder ohne Transparenz


                ' XOR-Daten-Array - enthlt Farbwerte
                ' Die XOR-Breite (XorStride) ist grer als die reale Breite der Bitmap 
                ' und muss eine Potenz von 2 sein
                Dim XorStride As Integer

                ' an dieser Stelle wird die indizierte Bitmap erstellt
                If BitsPerPixel = 8 Then
                    m_BMP = New Bitmap(Width, Width, Imaging.PixelFormat.Format8bppIndexed)
                    XorStride = (Width + 3) And Not 3
                Else ' (BitsPerPixel = 4) - es sind nur diese zwei Varianten notwendig
                    m_BMP = New Bitmap(Width, Width, Imaging.PixelFormat.Format4bppIndexed)
                    XorStride = ((Width + 7) And Not 7) \ 2
                End If

                ' zum Erstellen einer indizierten Farb-Palette habe ich eine separate Klasse vorbereitet:
                Dim Indexer As New ColorIndexer(ResultColors, m_BMP.Palette.Entries)

                ' dies wren dann die Farbindizes fr jeden einzelnen Pixel
                Dim Indizes() As Byte = Indexer.Indizes

                ' Xor-Array (enthlt Farbwerte) erstellen
                Dim m_XOR(XorStride * Width - 1) As Byte


                'temporre Positionen fr den Schleifendurchlauf in Folgenden
                Dim PosXor As Integer
                Dim Height As Integer = Width

                Select Case BitsPerPixel
                    Case 8 ' Format8bppIndexed
                        'fr 8 Bit pro Pixel (indizierte Farben):
                        For y As Integer = 0 To Height - 1
                            PosXor = XorStride * y
                            Pos32 = Width * y

                            For x As Integer = 0 To Width - 1
                                m_XOR(PosXor) = Indizes(Pos32)
                                PosXor += 1
                                Pos32 += 1
                            Next
                        Next
                    Case 4 ' Format4bppIndexed
                        'fr 4 Bit pro Pixel (indizierte Farben):
                        For y As Integer = 0 To Height - 1
                            PosXor = XorStride * y
                            Pos32 = Width * y
                            For x As Integer = 0 To Width - 1
                                Dim xPosXor As Integer = (x >> 1) + PosXor
                                If (x And 1) = 0 Then
                                    m_XOR(xPosXor) = m_XOR(xPosXor) Or (Indizes(Pos32) << 4)
                                Else
                                    m_XOR(xPosXor) = m_XOR(xPosXor) Or Indizes(Pos32)
                                End If
                                Pos32 += 1
                            Next
                        Next
                End Select

                ' An dieser Stelle sei daran erinnert, dass hier nur eine indizierte Bitmap 
                ' als Resultat gebraucht wird (entweder 8 oder 4 Bits pro Pixel)

                ' diese Bitmap wurde schon weiter oben mit entsprechendem Pixelformat 
                ' und Kantenlnge erstellt

                ' Nun werden in diese Bitmap die ermittelten XOR-Daten 
                ' (die XOR-Daten sind hier eine Reihe Farbindizes) kopiert
                Dim rect As New Rectangle(0, 0, Width, Height)
                Dim tmpbmpData As System.Drawing.Imaging.BitmapData = m_BMP.LockBits(rect, Drawing.Imaging.ImageLockMode.ReadWrite, m_BMP.PixelFormat)

                Dim xptr As IntPtr
                xptr = tmpbmpData.Scan0

                ' die XOR-Daten in die Bitmap kopieren
                System.Runtime.InteropServices.Marshal.Copy(m_XOR, 0, xptr, m_XOR.Length)

                ' Bits frei geben
                m_BMP.UnlockBits(tmpbmpData)

                ' wenn es einen transparenten Farbindex gibt, wird auch die entsprechende Farbe 
                ' in der Palette transparent
                If Indexer.TransparentColorIndex > -1 Then
                    Dim NewPalette As ColorPalette
                    NewPalette = m_BMP.Palette
                    NewPalette.Entries(Indexer.TransparentColorIndex) = Color.FromArgb(0, 0, 0, 0)
                    m_BMP.Palette = NewPalette
                End If
            End Using
        End Using

    End Sub
#End Region

End Class
