Public Class IconImageFormat
    Implements IComparable(Of IconImageFormat)

#Region " Felder "
    ' Breite
    Private m_Width As Integer
    ' Hhe
    Private m_Height As Integer
    ' Farbtiefe
    Private m_BitsPerPixel As Integer
    ' wenn PNG, dann True
    Private m_IsPNG As Boolean
#End Region

#Region " Properties "
    ''' <summary>
    ''' Breite in Pixeln
    ''' </summary>
    Overridable Property Width() As Integer
        Get
            Return m_Width
        End Get
        Set(ByVal value As Integer)
            m_Width = value
        End Set
    End Property
    ''' <summary>
    ''' Hhe in Pixeln
    ''' </summary>
    Overridable Property Height() As Integer
        Get
            Return m_Height
        End Get
        Set(ByVal value As Integer)
            m_Height = value
        End Set
    End Property
    ''' <summary>
    ''' Farbtiefe
    ''' </summary>
    Property BitsPerPixel() As Integer
        Get
            Return m_BitsPerPixel
        End Get
        Set(ByVal value As Integer)
            m_BitsPerPixel = value
        End Set
    End Property
    ''' <summary>
    ''' gibt an oder legt fest, ob das komprimierte PNG-Format verwendet wird
    ''' </summary>
    Property IsPNG() As Boolean
        Get
            Return m_IsPNG
        End Get
        Set(ByVal value As Boolean)
            m_IsPNG = value
        End Set
    End Property
#End Region

#Region " Konstruktoren "
    Public Sub New()

    End Sub
    Public Sub New(ByVal ImageWidth As Integer, ByVal ImageHeight As Integer, ByVal BitsPerPixel As Integer, ByVal IsPNG As Boolean)
        m_Width = ImageWidth
        m_Height = ImageHeight
        m_BitsPerPixel = BitsPerPixel
        m_IsPNG = IsPNG
    End Sub
#End Region

#Region " Methoden "
    ''' <summary>
    ''' vergleicht zwei IconImageFormat-Instanzen
    ''' </summary>
    Public Function CompareTo(ByVal other As IconImageFormat) As Integer Implements System.IComparable(Of IconImageFormat).CompareTo
        If other.IsPNG = Me.IsPNG Then
            If other.Width = Me.Width Then
                If other.Height = Me.Height Then
                    If Me.BitsPerPixel = other.BitsPerPixel Then
                        Return 0
                    Else
                        Return other.BitsPerPixel.CompareTo(Me.BitsPerPixel)
                    End If
                Else
                    Return other.Height.CompareTo(Me.Height)
                End If
            Else
                Return other.Width.CompareTo(Me.Width)
            End If

        Else
            Return other.IsPNG.CompareTo(Me.IsPNG)
        End If

    End Function

    ''' <summary>
    ''' liefert einen String, der das IconImageFormat beschreibt
    ''' </summary>
    Public Overrides Function ToString() As String
        Dim ZText As String = m_Width.ToString & " x " & m_Height.ToString & vbCrLf
        Select Case m_BitsPerPixel
            Case 1
                ZText &= "2 Farben"
            Case 4
                ZText &= "16 Farben"
            Case 8
                ZText &= "256 Farben"
            Case 16
                ZText &= "65536 Farben"
            Case 24
                ZText &= "1,6 Mio. Farben"
            Case 32, 64
                ZText &= "XP Farben"
            Case Else
                ZText &= "Unbekannte Farben"
        End Select

        If m_IsPNG Then
            ZText &= " (PNG)"
        End If
        Return ZText
    End Function
#End Region

End Class

