''' <summary>
''' Liste mit IconImages, die automatisch dafr sorgt, dass immer ein IconImage SelectedItem ist
''' und bei nderungen der Liste oder von SelectedItem Events liefert
''' </summary>
Public Class IconImageList
    Inherits List(Of IconImage)

    ' tritt ein, wenn sich die Anzahl der ListItems (und ggf. auch SelectedItem) gendert hat
    Public Event Changed()
    ' tritt ein, wenn nur SelectedItem gendert wurde
    Public Event SelectedItemChanged()
    Private m_SelectedItem As IconImage = Nothing
    Private m_Comparer As IconImageFormatComparer

    Public Sub New()
        m_Comparer = New IconImageFormatComparer
    End Sub

    Public Shadows Sub Add(ByVal item As IconImage)
        MyBase.Add(item)
        m_SelectedItem = item
        Me.Sort(m_Comparer)
        RaiseEvent Changed()
    End Sub
    Public Shadows Sub AddRange(ByVal collection As IEnumerable(Of IconImage))
        Dim TmpListe As New List(Of IconImage)
        TmpListe.AddRange(collection)
        TmpListe.Sort(m_Comparer)
        MyBase.AddRange(collection)
        Me.Sort(m_Comparer)
        If TmpListe.Count > 0 Then
            m_SelectedItem = TmpListe(0)
            RaiseEvent Changed()
        End If
    End Sub
    Public Shadows Sub Clear()
        MyBase.Clear()
        m_SelectedItem = Nothing
        RaiseEvent Changed()
    End Sub
    Public Shadows Sub Insert(ByVal index As Integer, ByVal item As IconImage)
        Add(item)
    End Sub
    Public Shadows Sub InsertRange(ByVal index As Integer, ByVal collection As IEnumerable(Of IconImage))
        AddRange(collection)
    End Sub
    Public Shadows Function Remove(ByVal item As IconImage) As Boolean
        Dim Index As Integer = Me.IndexOf(item)
        If Index > -1 Then
            Dim result As Boolean = MyBase.Remove(item)
            If Index > 0 Then
                m_SelectedItem = Me(Index - 1)
            ElseIf Me.Count > 0 Then
                m_SelectedItem = Me(0)
            Else
                m_SelectedItem = Nothing
            End If
            RaiseEvent Changed()
            Return result
        End If
        Return False
    End Function
    Public Shadows Sub RemoveAt(ByVal index As Integer)
        MyBase.RemoveAt(index)
        If index > 0 Then
            m_SelectedItem = Me(index - 1)
        ElseIf Me.Count > 0 Then
            m_SelectedItem = Me(0)
        Else
            m_SelectedItem = Nothing
        End If
        RaiseEvent Changed()
    End Sub


    ''' <summary>
    ''' Ausgewhltes Item
    ''' </summary>
    Property SelectedItem() As IconImage
        Get
            Return m_SelectedItem
        End Get
        Set(ByVal value As IconImage)
            If m_SelectedItem IsNot value Then
                If value Is Nothing OrElse Me.Contains(value) Then
                    m_SelectedItem = value
                    RaiseEvent SelectedItemChanged()
                End If
            End If
        End Set
    End Property

    ''' <summary>
    ''' Ausgewhlter Index
    ''' </summary>
    Property SelectedIndex() As Integer
        Get
            If m_SelectedItem Is Nothing Then
                Return -1
            Else
                Return Me.IndexOf(m_SelectedItem)
            End If
        End Get
        Set(ByVal value As Integer)
            Dim NewSelected As IconImage = Nothing
            If value > -1 AndAlso value < Me.Count Then
                NewSelected = Me(value)
            End If
            SelectedItem = NewSelected
        End Set
    End Property


    Private Class IconImageFormatComparer
        Implements IComparer(Of IconImage)
        Public Function Compare(ByVal x As IconImage, ByVal y As IconImage) As Integer Implements System.Collections.Generic.IComparer(Of IconImage).Compare
            Return x.CompareTo(y)
        End Function
    End Class
End Class
