''' <summary>
''' Bitmap information header (DIB header)
''' </summary>
''' <remarks>Bestandteil jeder Bitmap-Datei,
''' liefert wesentliche Informationen zum Inhalt der Bitmap</remarks>
Public Class BitmapInfoHeader

#Region " Private Felder "

    Private m_Size As UInteger = 40      ' Gre dieses Headers (40 Byte)
    Private m_Width As Integer           ' Breite der Bitmap in Pixeln
    Private m_Height As Integer          ' Hhe der Bitmap in Pixeln, im Icon der doppelten Hhe entsprechend
    Private m_Planes As UShort = 1       ' Anzahl Farb-Planes (kann ich nicht erklren)
    Private m_BitCount As UShort         ' Bits pro Pixel (Farbtiefe). Typische Werte sind 1, 4, 8, 16, 24 und 32
    Private m_Compression As UInteger    ' verwendete Komprimierungs-Methode
    Private m_SizeImage As UInteger      ' Lnge des Bitmap-Daten-Abschnitts
    Private m_XPelsPerMeter As Integer   ' Horizontale Auflsung der Bitmap in Pixel pro Meter
    Private m_YPelsPerMeter As Integer   ' Vertikale Auflsung der Bitmap in Pixel pro Meter
    Private m_ClrUsed As UInteger        ' Anzahl Farben in der Palette oder 0 
    Private m_ClrImportant As UInteger   ' Anzahl wichtiger Farben in der Palette oder 0, wenn alle Farben wichtig sind

#End Region

#Region " Properties "
    ''' <summary>
    ''' Breite der Bitmap in Pixeln
    ''' </summary>
    Property Width() As Integer
        Get
            Return m_Width
        End Get
        Set(ByVal value As Integer)
            m_Width = value
        End Set
    End Property

    ''' <summary>
    ''' Hhe der Bitmap in Pixeln, im Icon doppelte Hhe
    ''' </summary>
    Property Height() As Integer
        Get
            Return m_Height
        End Get
        Set(ByVal value As Integer)
            m_Height = value
        End Set
    End Property

    ''' <summary>
    ''' Bits pro Pixel (Farbtiefe)
    ''' </summary>
    Property BitCount() As UShort
        Get
            Return m_BitCount
        End Get
        Set(ByVal value As UShort)
            m_BitCount = value
        End Set

    End Property

    ''' <summary>
    ''' Anzahl Farben in der Palette
    ''' </summary>
    Property ClrUsed() As UInteger
        Get
            Return m_ClrUsed
        End Get
        Set(ByVal value As UInteger)
            m_ClrUsed = value
        End Set
    End Property
#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor mit einem Byte-Array 
    ''' </summary>
    Public Sub New(ByVal Bytes() As Byte)
        m_Size = BitConverter.ToUInt32(Bytes, 0)
        m_Width = BitConverter.ToInt32(Bytes, 4)
        m_Height = BitConverter.ToInt32(Bytes, 8)

        m_Planes = BitConverter.ToUInt16(Bytes, 12)
        m_BitCount = BitConverter.ToUInt16(Bytes, 14)

        m_Compression = BitConverter.ToUInt32(Bytes, 16)

        m_SizeImage = BitConverter.ToUInt32(Bytes, 20)
        m_XPelsPerMeter = BitConverter.ToInt32(Bytes, 24)
        m_YPelsPerMeter = BitConverter.ToInt32(Bytes, 28)
        m_ClrUsed = BitConverter.ToUInt32(Bytes, 32)
        m_ClrImportant = BitConverter.ToUInt32(Bytes, 36)
    End Sub

    ''' <summary>
    ''' Konstruktor mit fest stehenden Werten fr wesentliche Eigenschaften
    ''' </summary>
    ''' <param name="Width">Breite der Bitmap in Pixeln</param>
    ''' <param name="Height">Hhe der Bitmap in Pixeln, im Icon doppelte Hhe in Pixeln</param>
    ''' <param name="ColorCount">Anzahl Farben in der Palette</param>
    ''' <param name="BitCount">Bits pro Pixel (Farbtiefe)</param>
    ''' <param name="SizeImage">Lnge des Bitmap-Daten-Abschnitts in Byte</param>
    Public Sub New(ByVal Width As Integer, ByVal Height As Integer, ByVal ColorCount As Integer, ByVal BitCount As Integer, ByVal SizeImage As Integer)
        m_Width = Width
        m_Height = Height
        m_ClrUsed = ColorCount
        m_BitCount = BitCount
        m_SizeImage = SizeImage
    End Sub
#End Region

#Region " Methoden "
    ''' <summary>
    ''' schreibt den Inhalt des BitmapInfoHeader mit Hilfe eines BinaryWriter in einen Stream
    ''' </summary>
    ''' <param name="writer">BinaryWriter</param>
    Public Sub Write(ByVal writer As IO.BinaryWriter)

        writer.Write(m_Size)
        writer.Write(m_Width)
        writer.Write(m_Height)

        writer.Write(m_Planes)
        writer.Write(m_BitCount)

        writer.Write(m_Compression)

        writer.Write(m_SizeImage)
        writer.Write(m_XPelsPerMeter)
        writer.Write(m_YPelsPerMeter)
        writer.Write(m_ClrUsed)
        writer.Write(m_ClrImportant)
    End Sub
#End Region

End Class
