''' <summary>
''' Abschnitt einer Icon-Datei, der wesentliche Eigenschaften fr ein Einzelbild des Icons enthlt
''' </summary>
''' <remarks>Ein IconDirectory-Abschnitt ist 16 Bytes lang und enthlt u.a. Informationen 
''' zu Breite, Hhe, Farbtiefe Position und Lnge (in Bytes) eines einzelnen IconImage-Abschnitts</remarks>
Public Class IconDirectory
    ' 
#Region " Private Felder "

    Private m_Width As Byte             ' Breite des Bildes in Pixeln (wenn > 256, steht hier "0") 
    Private m_Height As Byte            ' Hhe des Bildes in Pixeln (wenn > 256, steht hier "0") 
    Private m_ColorCount As Byte        ' Anzahl Farben der Farbpalette (wenn > 256, steht hier "0") 
    Private m_Reserved As Byte = 0      ' Reserviert, ist immer 0 
    Private m_Planes As UShort = 1      ' Anzahl Farbplanes, ist immer 1 
    Private m_BitCount As UShort        ' Bits per Pixel (Farbtiefe) fr das Bild. bliche Werte sind 32, 8, 4 
    Private m_BytesInRes As UInteger    ' Lnge des zugehrigen IconImage-Abschnitts in Byte 
    Private m_ImageOffset As UInteger   ' Start-Position des zugehrigen IconImage-Abschnitts (Byte) in der Icon-Datei

#End Region

#Region " Properties "

    ''' <summary>
    ''' Lnge des zugehrigen IconImage-Abschnitts in Byte 
    ''' </summary>
    Public Property BytesInRes() As UInteger
        Get
            Return m_BytesInRes
        End Get
        Set(ByVal value As UInteger)
            m_BytesInRes = value
        End Set
    End Property
    ''' <summary>
    ''' Start-Position des zugehrigen IconImage-Abschnitts (Byte) in der Icon-Datei
    ''' </summary>
    Public Property ImageOffset() As UInteger
        Get
            Return m_ImageOffset
        End Get
        Set(ByVal value As UInteger)
            m_ImageOffset = value
        End Set
    End Property
    ''' <summary>
    ''' Bits per Pixel (Farbtiefe)
    ''' </summary>
    Property BitCount() As UShort
        Get
            Return m_BitCount
        End Get
        Set(ByVal value As UShort)
            m_BitCount = value
        End Set
    End Property

#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor fr einen BinaryReader zum Lesen des IconDirectory aus einem Stream
    ''' </summary>
    ''' <param name="Reader">BinaryReader</param>
    Public Sub New(ByVal Reader As IO.BinaryReader)
        Dim Bytes(15) As Byte
        Reader.Read(Bytes, 0, 16)

        m_Width = Bytes(0)
        m_Height = Bytes(1)
        m_ColorCount = Bytes(2)
        m_Reserved = Bytes(3)
        m_Planes = BitConverter.ToUInt16(Bytes, 4)
        m_BitCount = BitConverter.ToUInt16(Bytes, 6)
        m_BytesInRes = BitConverter.ToUInt32(Bytes, 8)
        m_ImageOffset = BitConverter.ToUInt32(Bytes, 12)
    End Sub
    ''' <summary>
    ''' Konstruktor mit fest stehenden Werten fr wesentliche Eigenschaften
    ''' </summary>
    ''' <param name="ImageWidth">Breite des Bildes in Pixeln</param>
    ''' <param name="ImageHeight">Hhe des Bildes in Pixeln</param>
    ''' <param name="ColorCount">Anzahl Farben der Farbpalette</param>
    ''' <param name="BitmapdataSize">Lnge des zugehrigen IconImage-Abschnitts in Byte</param>
    ''' <param name="BitmapDataPosition">Start-Position des zugehrigen IconImage-Abschnitts (Byte) in der Icon-Datei</param>
    ''' <param name="Bitcount">Bits per Pixel (Farbtiefe)</param>
    Public Sub New(ByVal ImageWidth As Integer, ByVal ImageHeight As Integer, ByVal ColorCount As Integer, _
    ByVal BitmapdataSize As Integer, ByVal BitmapDataPosition As Integer, ByVal Bitcount As Integer)

        If ImageWidth < 256 Then m_Width = ImageWidth
        If ImageHeight < 256 Then m_Height = ImageHeight
        If ColorCount < 256 Then m_ColorCount = ColorCount

        m_BitCount = CUShort(Bitcount)
        m_BytesInRes = CUInt(BitmapdataSize)
        m_ImageOffset = CUInt(BitmapDataPosition)
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' schreibt den Inhalt des IconDirectory mit Hilfe eines BinaryWriters in einen Stream
    ''' </summary>
    Public Sub Write(ByVal Writer As IO.BinaryWriter)
        Writer.Write(m_Width)
        Writer.Write(m_Height)
        Writer.Write(m_ColorCount)
        Writer.Write(m_Reserved)
        Writer.Write(m_Planes)
        Writer.Write(m_BitCount)
        Writer.Write(m_BytesInRes)
        Writer.Write(m_ImageOffset)
    End Sub

#End Region

End Class
