Imports System.Drawing.Imaging
''' <summary>
''' Auswahl-Dialog fr das Format eines neuen IconImage
''' </summary>
Public Class frm_NewLayer

#Region " Felder "
    ' Flag fr Initialisierung der Form
    Private m_Init As Boolean = False
    ' Referenz zur aktuell verwendeten Liste mit bereits vorhandenen IconImages
    Private m_Images As IconImageList
    ' True, wenn aktuell selektiertes IconImage als Grundlage fr ein neues IconImage verwendet werden soll
    Private m_AktuellesBildAlsGrundlage As Boolean = True
    ' ausgewhltes IconImageFormat
    Private m_ResultFormat As IconImageFormat
#End Region

#Region " Properties "
    ''' <summary>
    ''' ausgewhltes IconImageFormat
    ''' </summary>
    ReadOnly Property ResultFormat() As IconImageFormat
        Get
            Return m_ResultFormat
        End Get
    End Property

    ''' <summary>
    ''' True, wenn aktuell selektiertes IconImage als Grundlage fr ein neues IconImage verwendet werden soll
    ''' </summary>
    ReadOnly Property AktuellesBildAlsGrundlage() As Boolean
        Get
            Return m_AktuellesBildAlsGrundlage
        End Get
    End Property
#End Region

#Region " Konstruktoren "
    Public Sub New(ByVal PreferredFormat As IconImageFormat, ByVal images As IconImageList, ByVal BackColor As Color)

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fgen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        m_Images = images

        ' Init-flag auf True setzen, um Eventbehandlung nach Werte-nderung in Controls zu unterbrechen
        m_Init = True

        ' Im folgenden alle Auswahl-Elemente entsprechend den Eigenschaften 
        ' in PreferredFormat einstellen
        Me.CheckBoxPNG.Checked = PreferredFormat.IsPNG

        Select Case PreferredFormat.Width
            Case 256
                Me.RadioButtonSize256.Checked = True
            Case 48
                Me.RadioButtonSize48.Checked = True
            Case 32
                Me.RadioButtonSize32.Checked = True
            Case 24
                Me.RadioButtonSize24.Checked = True
            Case 16
                Me.RadioButtonSize16.Checked = True
        End Select

        Select Case PreferredFormat.BitsPerPixel
            Case 32
                Me.RadioButtonFarben32Bit.Checked = True
            Case 8
                Me.RadioButtonFarben8Bit.Checked = True
            Case 4
                Me.RadioButtonFarben4Bit.Checked = True
        End Select

        If m_Images.SelectedItem IsNot Nothing Then
            ' wenn in der IconImageList ein IconImage selektiert ist, 
            ' wird in den Anwendungseinstellungen nachgesehen, welche 
            ' Einstellung fr "Aktuelles Bild als Grundlage" verwendet werden soll
            m_AktuellesBildAlsGrundlage = My.Settings.AktuellesBildAlsGrundlage
            Me.CheckBoxAktuellesBild.Visible = True
            Me.CheckBoxAktuellesBild.Checked = m_AktuellesBildAlsGrundlage
        Else
            ' wenn in der IconImageList gar kein Item selektiert ist 
            ' (das passiert nur, wenn die Liste wirklich leer ist)
            ' wird die CheckBox "Aktuelles Bild als Grundlage" gar nicht erst angezeigt
            m_AktuellesBildAlsGrundlage = False
            Me.CheckBoxAktuellesBild.Checked = False
            Me.CheckBoxAktuellesBild.Visible = False
        End If

        ' Init-Flag auf False setzen
        m_Init = False

        ' Ergebnis-Format neu erstellen
        RefreshResult()
        ' Hintergrundfarbe (eingentlich vom Renderer) bernehmen
        Me.BackColor = BackColor
    End Sub

#End Region

#Region " Methoden "
    Private Sub RadioButtonSize_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RadioButtonSize256.CheckedChanged, RadioButtonSize48.CheckedChanged, RadioButtonSize32.CheckedChanged, RadioButtonSize24.CheckedChanged, RadioButtonSize16.CheckedChanged
        If m_Init Then Exit Sub
        RefreshResult()
    End Sub

    Private Sub RadioButtonFarben_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RadioButtonFarben32Bit.CheckedChanged, RadioButtonFarben8Bit.CheckedChanged, RadioButtonFarben4Bit.CheckedChanged
        If m_Init Then Exit Sub
        RefreshResult()
    End Sub

    Private Sub CheckBoxAktuellesBild_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxAktuellesBild.CheckedChanged
        If m_Init Then Exit Sub
        My.Settings.AktuellesBildAlsGrundlage = CheckBoxAktuellesBild.Checked
        m_AktuellesBildAlsGrundlage = CheckBoxAktuellesBild.Checked
    End Sub

    ''' <summary>
    ''' ermittelt das als Resultat dieses Dialogs erwartete IconImageformat neu
    ''' </summary>
    Private Sub RefreshResult()

        Dim ResultWidth As Integer
        If Me.RadioButtonSize256.Checked Then
            ResultWidth = 256
        ElseIf Me.RadioButtonSize48.Checked Then
            ResultWidth = 48
        ElseIf Me.RadioButtonSize32.Checked Then
            ResultWidth = 32
        ElseIf Me.RadioButtonSize24.Checked Then
            ResultWidth = 24
        ElseIf Me.RadioButtonSize16.Checked Then
            ResultWidth = 16
        End If
        Dim ResultPixelDepth As Integer
        If Me.RadioButtonFarben32Bit.Checked Then
            ResultPixelDepth = 32
        ElseIf Me.RadioButtonFarben8Bit.Checked Then
            ResultPixelDepth = 8
        ElseIf Me.RadioButtonFarben4Bit.Checked Then
            ResultPixelDepth = 4
        End If
        m_ResultFormat = New IconImageFormat(ResultWidth, ResultWidth, ResultPixelDepth, CheckBoxPNG.Checked)

    End Sub

    Private Sub CheckBoxPNG_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxPNG.CheckedChanged
        If m_Init Then Exit Sub
        RefreshResult()
    End Sub

    ''' <summary>
    ''' Handler fr Click-Event des OK-Buttons
    ''' </summary>
    Private Sub ButtonOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonOK.Click

        ' Prfen, ob das ausgewhlte Format im Icon bereits existiert
        Dim FormatExists As Boolean = False
        For Each Img As IconImage In m_Images
            If Img.CompareTo(m_ResultFormat) = 0 Then
                FormatExists = True
                Exit For
            End If
        Next


        If FormatExists Then
            ' wenn das ausgewhlte Format im Icon bereits existiert,
            ' fragen ob das vorhandene Bild berschrieben werden soll
            Dim Result As DialogResult = MessageBox.Show("Es existiert bereits ein Bild mit dem ausgewhlten Format." & vbCrLf & "Soll es berschrieben werden?", My.Application.Manager.AnwendungsTitel, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question)
            Select Case Result
                Case Windows.Forms.DialogResult.Yes
                    ' Wenn berschreiben erlaubt, aktuellen Dialog mit Result OK schliessen
                    Me.DialogResult = Windows.Forms.DialogResult.OK
                    Me.Close()
                Case Windows.Forms.DialogResult.Cancel
                    ' Wenn abbrechen gewnscht, diesen Dialog offen lassen
                Case Windows.Forms.DialogResult.No
                    ' Wenn berschreiben nicht zugelassen, aktuellen Dialog mit Result Cancel schliessen
                    Me.DialogResult = Windows.Forms.DialogResult.Cancel
                    Me.Close()
            End Select
        Else
            ' wenn das ausgewhlte Format im Icon noch nicht existiert,
            ' kann dieser Dialog mit Result OK geschlossen werden
            Me.DialogResult = Windows.Forms.DialogResult.OK
            Me.Close()
        End If
    End Sub
#End Region

End Class